from pydal import Field
from pydal.validators import (
    IS_DATETIME,
    IS_IN_DB,
    IS_EMPTY_OR
)
from datetime import datetime


class GalleryTables():
    def __init__(self, DALDatabase):
        self.DALDatabase = DALDatabase
        self.DALDatabase.define_table('phanterpwagallery',
            Field('folder'),
            Field('filename'),
            Field('alias_name'),
            Field('content_type')
        )

        self.DALDatabase.define_table('auth_user_phanterpwagallery',
            Field('phanterpwagallery', 'reference phanterpwagallery'),
            Field('auth_user', 'reference auth_user', requires=IS_EMPTY_OR(
                IS_IN_DB(self.DALDatabase, self.DALDatabase.auth_user))),
            Field('subfolder'),
            Field('last_update', 'datetime', default=datetime.now(), requires=IS_EMPTY_OR(IS_DATETIME()))
        )



from core import projectConfig as CONFIG
from phanterpwa.components.preloaders import run_points
from phanterpwa.helpers import DIV

PRELOADER = run_points.PRELOADER
if CONFIG['PROJECT']["debug"]:
    import os
    from phanterpwa.tools import app_name_from_relative_child
    app_folder_name = app_name_from_relative_child(CONFIG['PROJECT']['path'], __file__)
    PRELOADER._target_sass = os.path.join(
        CONFIG["PROJECT"]["path"], "frontapps", app_folder_name, "sources", "styles", "extends", "run_points.sass")
PRELOADER.sass_vars = {
    'COLOR1': 'red',
    'COLOR2': 'blue',
    'COLOR3': 'yellow',
    'COLOR4': 'pink',
    'COLOR5': 'green',
    'BORDERRADIUS': '100%',
    'CONTAINERWIDTH': '100px'
}
html = DIV(PRELOADER, _class="container_preloader_button_auth_user")



from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = String = setTimeout =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0

__pragma__('noskip')
__pragma__('kwargs')

def zfill(number, size):
    number = int(number)
    number = str(number)
    s = number
    for x in range(size - len(number)):
        s = "0" + s
    return s


class Mask():
    def __init__(self, target_selector, mask_function, reverse=False, apply_on_init=False):
        self.target_selector = target_selector
        self.element_target = jQuery(target_selector)
        self.mask_function = mask_function
        self.reverse = reverse
        self.apply_on_init = apply_on_init
        self.start()

    @staticmethod
    def stringFilter(
        value,
        you_want_array=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]):
        value = str(value)
        new_value = ""
        for x in value:
            if x in you_want_array:
                new_value += x
        return new_value

    def onKeyPress(self, event, el):
        event.preventDefault()
        code = event.keyCode or event.which
        element = jQuery(el)
        pos = element[0].selectionStart
        end = element[0].selectionEnd
        if pos == end:
            current_value = element.val()
            v = String.fromCharCode(code)
            text0 = current_value[0: pos] + v
            text1 = current_value[pos:]
            numbers = [str(x) for x in range(10)]
            if v in numbers:
                # print(current_value[pos])
                if current_value[pos] in numbers or current_value[pos] == "_":
                    pos = pos + 1
                else:
                    pos = pos + 2
        else:
            current_value = element.val()
            v = String.fromCharCode(code)
            text0 = current_value[0: pos] + v
            text1 = current_value[end:]
            numbers = [str(x) for x in range(10)]
            if v in numbers:
                pos = pos + 1
        new_value = "{0}{1}".format(text0, text1)
        pure_value = self.stringFilter(new_value)
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")

        element[0].selectionStart = self.mask_function(pure_value)[1]
        element[0].selectionEnd = self.mask_function(pure_value)[1]

    def onKeyUp(self, event, el):
        element = jQuery(el)
        new_value = element.val()
        pure_value = self.stringFilter(new_value)
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")

        element[0].selectionStart = self.mask_function(pure_value)[1]
        element[0].selectionEnd = self.mask_function(pure_value)[1]
        element.focus()
        element[0].setSelectionRange(self.mask_function(pure_value)[1], self.mask_function(pure_value)[1])


    def onNonPrintingKeysIn(self, event, el):
        noprintkeys = [8, 46]
        code = event.keyCode or event.which
        element = jQuery(el)
        if code in noprintkeys:
            if code == 8:
                current_value = element.val()
                if self.stringFilter(current_value) is not "":
                    pure_value = self.stringFilter(current_value)
                    pure_value = pure_value[0:-1]
                    new_value = self.mask_function(pure_value)[0]
                    if pure_value is not "":
                        element.val(new_value)
                    else:
                        element.val("")
                    element[0].selectionStart = self.mask_function(pure_value)[1]
                    element[0].selectionEnd = self.mask_function(pure_value)[1]
                else:
                    element.val("")
            elif code == 46:
                current_value = element.val()
                if self.stringFilter(current_value) is not "":
                    pos = element[0].selectionStart
                    end = element[0].selectionEnd
                    if pos == end:
                        text0 = current_value[0:pos]
                        numbers = [str(x) for x in range(10)]
                        if current_value[pos] in numbers:
                            text1 = current_value[pos + 1:]
                        elif current_value[pos] is not "":
                            text1 = current_value[pos + 2:]

                        new_value = "{0}{1}".format(text0, text1)
                        element[0].selectionStart = pos
                        element[0].selectionEnd = pos
                    else:
                        text0 = current_value[0: pos]
                        text1 = current_value[end:]
                        new_value = "{0}{1}".format(text0, text1)
                        element[0].selectionStart = pos
                        element[0].selectionEnd = pos
                    pure_value = self.stringFilter(new_value)
                    new_value = self.mask_function(pure_value)[0]
                    if pure_value is not "":
                        element.val(new_value)
                    else:
                        element.val("")
                    element[0].selectionStart = pos
                    element[0].selectionEnd = pos
                else:
                    element.val("")

            event.preventDefault()

    def onNonPrintingKeys(self, event, el):
        event.preventDefault()
        element = jQuery(el)
        code = event.keyCode or event.which
        noprintkeys = [8, 46, 9]
        if code in noprintkeys:
            value = element.val()
            element.val(value + "_")
            if (self.reverse):
                if (self.stringFilter(value) != ""):
                    value = str(int(self.stringFilter(value)))
            new_value = ""

            pure_value = self.stringFilter(value)
            if pure_value == "":
                element.val("")
            else:
                new_value = self.mask_function(pure_value)[0]
                element.attr("phanterpwa-mask-justnumbers", self.stringFilter(new_value))
                selection_pos = self.mask_function(pure_value)[1]
                element.val(new_value)
                if (self.reverse):
                    element[0].selectionStart = -len(new_value)
                    element[0].selectionEnd = -len(new_value)
                else:
                    element[0].selectionStart = selection_pos
                    element[0].selectionEnd = selection_pos
                if(code != 9):
                    event.preventDefault()
        else:
            pure_value = self.stringFilter(element.val())
            if pure_value == "":
                element.val("")

    def onFocusOut(self, event, el):
        element = jQuery(el)
        new_value = element.val()
        pure_value = self.stringFilter(new_value)
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")


    def start(self):
        element = jQuery(self.target_selector)
        value = element.val()
        pure_value = self.stringFilter(value)
        new_value = self.mask_function(pure_value)[0]
        selection_pos = self.mask_function(pure_value)[1]

        if(self.apply_on_init):
            element.val(new_value)
            if(self.reverse):
                element[0].selectionStart = -len(new_value)
                element[0].selectionEnd = -len(new_value)
            else:
                element[0].selectionStart = selection_pos
                element[0].selectionEnd = selection_pos

        element.off(
            "keypress.phanterpwaMask"
        ).on(
            "keypress.phanterpwaMask",
            lambda event: self.onKeyPress(event, this)
        )
        element.off(
            "focusout.phanterpwaMask"
        ).on(
            "focusout.phanterpwaMask",
            lambda event: self.onFocusOut(event, this)
        )
        element.off(
            "keyup.phanterpwaMask2,"
        ).on(
            "keyup.phanterpwaMask2,",
            lambda event: self.onKeyUp(event, this)
        )
        element.off(
            "keydown.phanterpwaMask, focusout.phanterpwaMask"
        ).on(
            "keydown.phanterpwaMask, focusout.phanterpwaMask",
            lambda event: self.onNonPrintingKeysIn(event, this)
        )


class Currency():
    def __init__(self, target_selector, reverse=False, apply_on_init=False, casas_decimais=2, separador_decimal=",", separador_milhar=".", icurrency=""):
        self.target_selector = target_selector
        self.element_target = jQuery(target_selector)
        self.reverse = reverse
        self.casas_decimais = casas_decimais
        self.apply_on_init = apply_on_init
        self.separador_milhar = separador_milhar
        self.separador_decimal = separador_decimal
        self.icurrency = icurrency
        self.start()

    def mask_function(self, value):
        value = self.stringFilter(str(value))
        if len(value) <= self.casas_decimais:
            nvalue = "0{}{}".format(self.separador_decimal, zfill(value, 2))
        else:
            ivalue = int(value[0: -self.casas_decimais])
            p_inteiro = str(ivalue).split("").reverse().join("")
            str_inteiro = ""
            tamanho_inteiro = len(p_inteiro)
            adicionar_separador = False
            if (tamanho_inteiro > 3):
                for i in range(tamanho_inteiro):
                    if (((i + 1) % 3) == 0):
                        str_inteiro += p_inteiro[i]
                        adicionar_separador = True
                    else:
                        if(adicionar_separador):
                            adicionar_separador = False
                            str_inteiro += self.separador_milhar + p_inteiro[i]
                        else:
                            str_inteiro += p_inteiro[i]
                str_inteiro = str_inteiro.split("").reverse().join("")
            else:
                str_inteiro = str(ivalue)
            nvalue = "{}{}{}".format(
                str_inteiro,
                self.separador_decimal,
                value[-int(self.casas_decimais):]
            )
        if self.icurrency != "":
            nvalue = "{} {}".format(self.icurrency, nvalue)
        return [nvalue, len(nvalue)]

    @staticmethod
    def stringFilter(
        value,
        you_want_array=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]):
        value = str(value)
        new_value = ""
        for x in value:
            if x in you_want_array:
                new_value += x
        return new_value

    def onKeyPress(self, event, el):
        event.preventDefault()
        code = event.keyCode or event.which
        element = jQuery(el)
        pos = element[0].selectionStart
        end = element[0].selectionEnd
        if pos == end:
            current_value = element.val()
            v = String.fromCharCode(code)
            text0 = current_value[0: pos] + v
            text1 = current_value[pos:]
            numbers = [str(x) for x in range(10)]
            if v in numbers:
                # print(current_value[pos])
                if current_value[pos] in numbers or current_value[pos] == "_":
                    pos = pos + 1
                else:
                    pos = pos + 2
        else:
            current_value = element.val()
            v = String.fromCharCode(code)
            text0 = current_value[0: pos] + v
            text1 = current_value[end:]
            numbers = [str(x) for x in range(10)]
            if v in numbers:
                pos = pos + 1
        new_value = "{0}{1}".format(text0, text1)
        pure_value = self.stringFilter(new_value, ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", self.separador_decimal])
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")

        element[0].selectionStart = self.mask_function(pure_value)[1]
        element[0].selectionEnd = self.mask_function(pure_value)[1]

    def onKeyUp(self, event, el):
        element = jQuery(el)
        new_value = element.val()
        pure_value = self.stringFilter(new_value)
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")

        element[0].selectionStart = self.mask_function(pure_value)[1]
        element[0].selectionEnd = self.mask_function(pure_value)[1]
        element.focus()
        element[0].setSelectionRange(self.mask_function(pure_value)[1], self.mask_function(pure_value)[1])


    def onNonPrintingKeysIn(self, event, el):
        noprintkeys = [8, 46]
        code = event.keyCode or event.which
        element = jQuery(el)
        if code in noprintkeys:
            if code == 8:
                current_value = element.val()
                if self.stringFilter(current_value) is not "":
                    pure_value = self.stringFilter(current_value)
                    pure_value = pure_value[0:-1]
                    new_value = self.mask_function(pure_value)[0]
                    if pure_value is not "":
                        element.val(new_value)
                    else:
                        element.val("")
                    element[0].selectionStart = self.mask_function(pure_value)[1]
                    element[0].selectionEnd = self.mask_function(pure_value)[1]
                else:
                    element.val("")
            elif code == 46:
                current_value = element.val()
                if self.stringFilter(current_value) is not "":
                    pos = element[0].selectionStart
                    end = element[0].selectionEnd
                    if pos == end:
                        text0 = current_value[0:pos]
                        numbers = [str(x) for x in range(10)]
                        if current_value[pos] in numbers:
                            text1 = current_value[pos + 1:]
                        elif current_value[pos] is not "":
                            text1 = current_value[pos + 2:]

                        new_value = "{0}{1}".format(text0, text1)
                        element[0].selectionStart = pos
                        element[0].selectionEnd = pos
                    else:
                        text0 = current_value[0: pos]
                        text1 = current_value[end:]
                        new_value = "{0}{1}".format(text0, text1)
                        element[0].selectionStart = pos
                        element[0].selectionEnd = pos
                    pure_value = self.stringFilter(new_value)
                    new_value = self.mask_function(pure_value)[0]
                    if pure_value is not "":
                        element.val(new_value)
                    else:
                        element.val("")
                    element[0].selectionStart = pos
                    element[0].selectionEnd = pos
                else:
                    element.val("")

            event.preventDefault()

    def onNonPrintingKeys(self, event, el):
        event.preventDefault()
        element = jQuery(el)
        code = event.keyCode or event.which
        noprintkeys = [8, 46, 9]
        if code in noprintkeys:
            value = element.val()
            element.val(value + "_")
            if (self.reverse):
                if (self.stringFilter(value) != ""):
                    value = str(int(self.stringFilter(value)))
            new_value = ""

            pure_value = self.stringFilter(value)
            if pure_value == "":
                element.val("")
            else:
                new_value = self.mask_function(pure_value)[0]
                element.attr("phanterpwa-mask-justnumbers", self.stringFilter(new_value))
                selection_pos = self.mask_function(pure_value)[1]
                element.val(new_value)
                if (self.reverse):
                    element[0].selectionStart = -len(new_value)
                    element[0].selectionEnd = -len(new_value)
                else:
                    element[0].selectionStart = selection_pos
                    element[0].selectionEnd = selection_pos
                if(code != 9):
                    event.preventDefault()
        else:
            pure_value = self.stringFilter(element.val())
            if pure_value == "":
                element.val("")

    def onFocusOut(self, event, el):
        element = jQuery(el)
        new_value = element.val()
        pure_value = self.stringFilter(new_value)
        new_value = self.mask_function(pure_value)[0]
        if pure_value is not "":
            element.val(new_value)
        else:
            element.val("")


    def start(self):
        element = jQuery(self.target_selector)
        value = element.val()
        pure_value = self.stringFilter(value)
        new_value = self.mask_function(pure_value)[0]
        selection_pos = self.mask_function(pure_value)[1]

        if(self.apply_on_init):
            element.val(new_value)
            if(self.reverse):
                element[0].selectionStart = -len(new_value)
                element[0].selectionEnd = -len(new_value)
            else:
                element[0].selectionStart = selection_pos
                element[0].selectionEnd = selection_pos

        element.off(
            "keypress.phanterpwaMask"
        ).on(
            "keypress.phanterpwaMask",
            lambda event: self.onKeyPress(event, this)
        )
        element.off(
            "focusout.phanterpwaMask"
        ).on(
            "focusout.phanterpwaMask",
            lambda event: self.onFocusOut(event, this)
        )
        element.off(
            "keyup.phanterpwaMask2,"
        ).on(
            "keyup.phanterpwaMask2,",
            lambda event: self.onKeyUp(event, this)
        )
        element.off(
            "keydown.phanterpwaMask, focusout.phanterpwaMask"
        ).on(
            "keydown.phanterpwaMask, focusout.phanterpwaMask",
            lambda event: self.onNonPrintingKeysIn(event, this)
        )



def date_and_datetime_to_maks(value):
    date_format = ["d", "M", "o", "t", "y", "H", "m", "s"]
    string_mask = ""
    if isinstance(value, str):
        for x in value:
            if x in date_format:
                y = "#"
            else:
                y = x
            string_mask += y
    return string_mask


def isNotEmpty(value):
    if ((value is not None) and (value is not "") and (value is not js_undefined)):
        return True
    else:
        return False


def stringFilter(
    value,
    you_want_array=["0", "1", "2", "3", "4", "5", "6", "7", "8", "9"]):
    value = str(value)
    new_value = ""
    for x in value:
        if x in you_want_array:
            new_value += x
    return new_value


def hasCaracter(value, caracter="."):
    value = str(value)
    has_caractere = False
    if caracter in value:
        has_caractere = True
    return has_caractere


def justSearchedCaracter(value, caractere="."):
    has_caractere = False
    value = str(value)
    new_value = ""
    for x in value:
        if x == caractere:
            if not has_caractere:
                new_value += x
                has_caractere = True
        else:
            new_value += x
    return new_value


def stringForceToFloatstring(value, force_dot=False, localeBR=True):
    if isNotEmpty(value):
        value = str(value)
        if localeBR:
            value = value.replace(".", "")
            value = value.replace(",", ".")
        value = stringFilter(value, ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "."])
        value = justSearchedCaracter(value)
        if (value != ""):
            if (value == "."):
                if (force_dot):
                    value = "0."
                else:
                    value = ""
            else:
                if not force_dot:
                    value = str(float(value))
        return value
    else:
        return ""


def stringToFloatstringLimitDecimals(value, casas_decimais=2, localeBR=True):
    value = str(value)
    if isNotEmpty(value):
        if localeBR:
            value = value.replace(".", "")
            value = value.replace(",", ".")
        value = stringFilter(value, ["0", "1", "2", "3", "4", "5", "6", "7", "8", "9", "."])
        value = justSearchedCaracter(value)
        if(value != ""):
            value = str(float(value))
            p_inteiro = value.split(".")[0]
            p_decimal = value.split(".")[1]
            if not isNotEmpty(p_decimal):
                p_decimal = "0"
            if len(p_decimal) < casas_decimais:
                np_decimal = ""
                for x in range(casas_decimais - len(p_decimal)):
                    np_decimal += "0"
                p_decimal = p_decimal + np_decimal
            else:
                np_decimal = ""
                for i in range(casas_decimais):
                    np_decimal += p_decimal[i]
                p_decimal = np_decimal
            r = "{0}.{1}".format(p_inteiro, p_decimal)
            return r
        return value
    else:
        return ""


def floatToCurrency(value, casas_decimais=2, separador_decimal=",", separador_milhar=".", currency=""):
    casas_decimais = casas_decimais
    separador_decimal = separador_decimal
    separador_milhar = separador_milhar
    value = str(value)
    p_m_inteiro = "0"
    p_m_decimal = ""
    for i in range(casas_decimais):
        p_m_decimal += "0"

    if hasCaracter(value, "."):
        p_inteiro = value.split(".")[0]
        p_decimal = value.split(".")[1]
        if(isNotEmpty(p_inteiro)):
            p_inteiro = p_inteiro.split("").reverse().join("")
            str_inteiro = ""
            tamanho_inteiro = len(p_inteiro)
            adicionar_separador = False
            if (tamanho_inteiro > 3):
                for i in range(tamanho_inteiro):
                    if (((i + 1) % 3) == 0):
                        str_inteiro += p_inteiro[i]
                        adicionar_separador = True
                    else:
                        if(adicionar_separador):
                            adicionar_separador = False
                            str_inteiro += separador_milhar + p_inteiro[i]
                        else:
                            str_inteiro += p_inteiro[i]
            else:
                for i in range(tamanho_inteiro):
                    str_inteiro += p_inteiro[i]
            p_m_inteiro = str_inteiro.split("").reverse().join("")

        if(isNotEmpty(p_decimal)):
            str_cd = ""
            if (p_decimal.length > casas_decimais):
                for i in range(casas_decimais):
                    str_cd += p_decimal[i]
            else:
                for i in range(len(p_decimal)):
                    str_cd += p_decimal[i]
                diferenca = casas_decimais - len(p_decimal)
                tracos = ""
                for i in range(diferenca):
                    tracos += "0"
                str_cd += tracos
            p_m_decimal = str_cd

    else:
        if value != "":
            t_m_inteiro = int(value)
            if (isNotEmpty(t_m_inteiro)):
                p_m_inteiro = str(t_m_inteiro)
    if(isNotEmpty(currency)):
        r = "{0} {1}{2}{3}".format(currency, p_m_inteiro, separador_decimal, p_m_decimal)
    else:
        r = "{0}{1}{2}".format(p_m_inteiro, separador_decimal, p_m_decimal)
    return r


def baseCustom(value, custom_mask, cursorPosition=0):
    value = str(value)
    size = len(value)
    char_plus = 0
    pos_num = 0
    new_value = ""
    for i in range(len(custom_mask)):
        if (custom_mask[i] == "#"):
            if (pos_num < size):
                new_value += value[pos_num]
                pos_num += 1
            else:
                new_value += "_"
                pos_num += 1
        else:
            if (i < (size + char_plus)):
                char_plus += 1

            new_value += custom_mask[i]
    if new_value.indexOf("_") > 0:
        cursorPosition = new_value.indexOf("_")
    else:
        cursorPosition = int(size) + char_plus
    return [new_value, cursorPosition]


def maskFone(valor):
    valor = str(valor)
    size = len(valor)
    if (size == 10):
        custom_mask = "(##) ####-####"
    elif (size == 11):
        custom_mask = "(##) # ####-####"
    elif (size > 11):
        custom_mask = "(##) #####-#########"
    else:
        custom_mask = "(##) # ####-####"
    return baseCustom(valor, custom_mask)


def maskCNPJ(valor):
    custom_mask = "##.###.###/####-##"
    return baseCustom(valor, custom_mask)


def maskCPF(valor):
    custom_mask = "###.###.###-##"
    return baseCustom(valor, custom_mask)


def maskDate(valor):
    custom_mask = "##/##/####"
    return baseCustom(valor, custom_mask)


def maskDatetime(valor):
    custom_mask = "##/##/#### ##:##:##"
    return baseCustom(valor, custom_mask)


def maskCEP(valor):
    custom_mask = "##.###-###"
    return baseCustom(valor, custom_mask)


def applyMask(jq_select, maskfunction, reverse=False, apply_on_init=False):

    def onKeyPress(event, el):
        element = jQuery(el)
        code = event.keyCode or event.which
        noprintkeys = [8, 46, 9]
        if code not in noprintkeys:
            value = element.val()
            if (reverse):
                if (stringFilter(value) != ""):
                    value = str(int(stringFilter(value)))
            new_value = ""

            value = value + String.fromCharCode(code)
            pure_value = stringFilter(value)
            if pure_value == "":
                element.val("")
            else:
                new_value = maskfunction(pure_value)[0]
                element.attr("phanterpwa-mask-justnumbers", stringFilter(new_value))
                selection_pos = maskfunction(pure_value)[1]
                element.val(new_value)
                if (reverse):
                    element[0].selectionStart = -len(new_value)
                    element[0].selectionEnd = -len(new_value)
                else:
                    element[0].selectionStart = selection_pos
                    element[0].selectionEnd = selection_pos
                if(code != 9):
                    event.preventDefault()

    def onNonPrintingKeys(event, el):
        element = jQuery(el)
        code = event.keyCode or event.which
        noprintkeys = [8, 46, 9]
        if code in noprintkeys:
            value = element.val()
            if (reverse):
                if (stringFilter(value) != ""):
                    value = str(int(stringFilter(value)))
            new_value = ""

            pure_value = stringFilter(value)
            if pure_value == "":
                element.val("")
            else:
                new_value = maskfunction(pure_value)[0]
                element.attr("phanterpwa-mask-justnumbers", stringFilter(new_value))
                selection_pos = maskfunction(pure_value)[1]
                element.val(new_value)
                if (reverse):
                    element[0].selectionStart = -len(new_value)
                    element[0].selectionEnd = -len(new_value)
                else:
                    element[0].selectionStart = selection_pos
                    element[0].selectionEnd = selection_pos
                if(code != 9):
                    event.preventDefault()

    def onEachElement(el):
        element = jQuery(el)
        value = element.val()
        pure_value = stringFilter(value)
        new_value = maskfunction(pure_value)[0]
        selection_pos = maskfunction(pure_value)[1]

        if(apply_on_init):
            element.val(new_value)
            if(reverse):
                element[0].selectionStart = -len(new_value)
                element[0].selectionEnd = -len(new_value)
            else:
                element[0].selectionStart = selection_pos
                element[0].selectionEnd = selection_pos

        element.off(
            "keypress.phanterpwaMask, focusout.phanterpwaMask"
        ).on(
            "keypress.phanterpwaMask, focusout.phanterpwaMask",
            lambda event: onKeyPress(event, this)
        )

        element.off(
            "keyup.phanterpwaMask, focusout.phanterpwaMask"
        ).on(
            "keyup.phanterpwaMask, focusout.phanterpwaMask",
            lambda event: onNonPrintingKeys(event, this)
        )

    jQuery(jq_select).each(lambda: onEachElement(this))
    return jQuery(jq_select)


def phanterCurrency(
    jq_select,
    v_currency="R$",
    casas_decimais=2,
    separador_decimal=",",
    separador_milhar="."):
    return phanterDecimals(
        jq_select=jq_select,
        v_currency=v_currency,
        casas_decimais=casas_decimais,
        separador_decimal=separador_decimal,
        separador_milhar=separador_milhar)


def phanterDecimals(
    jq_select,
    v_currency="",
    casas_decimais=2,
    separador_decimal=",",
    separador_milhar="."):
    l_currency = v_currency

    def onKeyPress(event, el):
        element = jQuery(el)
        code = event.keyCode or event.which
        value = element.attr("phantermaskTemp")
        key_value = String.fromCharCode(code)
        if (code == 8):
            if(hasCaracter(value, ".")):
                value = stringToFloatstringLimitDecimals(value, casas_decimais)
            value = stringFilter(value)
            value = value[0:-1]
        elif((code == 46) and event.key == "Delete"):
            value = stringFilter(value)
            value = value[1:]
        else:
            contat_value = "{0}{1}".format(value, key_value)
            if (((key_value == separador_decimal) or (event.key == separador_decimal)) and not hasCaracter(value, ".")):
                value = stringForceToFloatstring(contat_value, True)
            else:
                if (hasCaracter(value, ".")):
                    value = stringForceToFloatstring(contat_value, True)
                else:
                    value = stringForceToFloatstring(contat_value)
        element.attr("phantermaskTemp", value)
        if not hasCaracter(value, "."):
            if (len(value) >= casas_decimais):
                p_inteiro = value[0: (-1) * casas_decimais]
                if not isNotEmpty(p_inteiro):
                    p_inteiro = "0"
                p_decimal = value[(-1) * casas_decimais:]
                value = "{0}.{1}".format(p_inteiro, p_decimal)
            else:
                diferenca = casas_decimais - len(value)
                add_decs = ""
                for i in range(diferenca):
                    add_decs += "0"
                p_inteiro = "0"
                value = "{0}.{1}{2}".format(p_inteiro, add_decs, value)
        value = stringToFloatstringLimitDecimals(value, casas_decimais)
        new_value = ""
        if (value == ""):
            qu_decs = ""
            for i in range(casas_decimais):
                qu_decs += "0"
            new_value = "0{0}{1}".format(separador_decimal, qu_decs)
            if(element.prop('TagName') == "INPUT"):
                element.val(
                    floatToCurrency(
                        stringToFloatstringLimitDecimals(
                            new_value,
                            casas_decimais
                        ),
                        casas_decimais,
                        separador_decimal,
                        separador_milhar,
                        l_currency)
                )
            else:
                element.text(
                    floatToCurrency(
                        stringToFloatstringLimitDecimals(
                            new_value,
                            casas_decimais
                        ),
                        casas_decimais,
                        separador_decimal,
                        separador_milhar,
                        l_currency)
                )
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(new_value, casas_decimais))
        else:
            new_value = value.replace(".", separador_decimal)
            if(element.prop('TagName') == "INPUT"):
                element.val(
                    floatToCurrency(
                        stringToFloatstringLimitDecimals(
                            new_value,
                            casas_decimais
                        ),
                        casas_decimais,
                        separador_decimal,
                        separador_milhar,
                        l_currency)
                )
            else:
                element.text(
                    floatToCurrency(
                        stringToFloatstringLimitDecimals(
                            new_value,
                            casas_decimais
                        ),
                        casas_decimais,
                        separador_decimal,
                        separador_milhar,
                        l_currency)
                )
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(new_value, casas_decimais))
        element[0].selectionStart = -len(new_value)
        element[0].selectionEnd = -len(new_value)
        if (code != 9):
            event.preventDefault()

    def onPaste(el):
        setTimeout(
            lambda: jQuery(el).trigger("focusout"),
            100
        )

    def onFocusOut(event, el):
        element = jQuery(el)
        value = element.attr("phantermaskValue")
        value = stringToFloatstringLimitDecimals(value, casas_decimais)
        new_value = ""
        if (value == ""):
            qu_decs = ""
            for i in range(casas_decimais):
                qu_decs += "0"
            new_value = "0{0}{1}".format(separador_decimal, qu_decs)
            if(element.prop('TagName') == "INPUT"):
                element.val(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            else:
                element.text(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(value, casas_decimais))
        else:
            new_value = value.replace(".", separador_decimal)
            if(element.prop('TagName') == "INPUT"):
                element.val(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            else:
                element.text(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(value, casas_decimais))
        element[0].selectionStart = -len(new_value)
        element[0].selectionEnd = -len(new_value)

    def onEachElement(el):
        l_currency = ""
        if(v_currency != ""):
            l_currency = v_currency
        element = jQuery(el)
        if(element.prop('TagName') == "INPUT"):
            defaults = element.val()
        else:
            defaults = element.text()
        value = stringForceToFloatstring(defaults)
        element.attr("phantermaskTemp", value)
        value = stringToFloatstringLimitDecimals(value, casas_decimais)
        new_value = ""
        if (value == ""):
            qu_decs = ""
            for i in range(casas_decimais):
                qu_decs += "0"
            new_value = "0" + separador_decimal + qu_decs
            if(element.prop('TagName') == "INPUT"):
                element.val(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            else:
                element.text(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(value, casas_decimais))
        else:
            new_value = value.replace(".", separador_decimal)
            if(element.prop('TagName') == "INPUT"):
                element.val(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            else:
                element.text(floatToCurrency(stringToFloatstringLimitDecimals(
                    new_value, casas_decimais), casas_decimais, separador_decimal, separador_milhar, l_currency))
            element.attr("phantermaskValue", stringToFloatstringLimitDecimals(value, casas_decimais))
        element[0].selectionStart = -len(new_value)
        element[0].selectionEnd = -len(new_value)
        element.off(
            'keypress.phanterpwaMask'
        ).on(
            'keypress.phanterpwaMask',
            lambda event: onKeyPress(event, this)
        ).off(
            "paste.phanterpwaMask, focusin.phanterpwaMask"
        ).on(
            "paste.phanterpwaMask, focusin.phanterpwaMask",
            lambda: onFocusOut(this)
        ).off(
            'focusout.phanterpwaMask'
        ).on(
            'focusout.phanterpwaMask',
            lambda event: onFocusOut(event, this)
        )
    jq_select.each(lambda: onEachElement(this))
    return jQuery(jq_select)


def phanterpwaMask(mask, parameters, el):
    custom_mask = ""
    casas_decimais = 2
    separador_decimal = ","
    separador_milhar = "."
    currency = "R$"
    reverse = False
    value = ""
    apply_on_init = False
    date_format = "%d/%m/%Y"
    datetime_format = "%d/%m/%Y %H:%M:%S"
    if (isNotEmpty(parameters)):
        if ("mask" in parameters):
            custom_mask = parameters['mask']
        if ("casas_decimais" in parameters):
            casas_decimais = int(parameters['casas_decimais'])
        if ("separador_decimal" in parameters):
            separador_decimal = str(parameters['separador_decimal'])
        if ("separador_milhar" in parameters):
            separador_milhar = str(parameters['separador_milhar'])
        if ("currency" in parameters):
            currency = str(parameters['currency'])
        if ("value" in parameters):
            jQuery(el).val(parameters['value'])
        if ("date_format" in parameters):
            date_format = parameters['date_format']
        if ("datetime_format" in parameters):
            datetime_format = parameters['datetime_format']
        if ("apply_on_init" in parameters):
            apply_on_init = parameters['apply_on_init']
    jQuery(el).removeClass("masked_input")
    if (mask == "fone"):
        jQuery(el).addClass("masked_input")
        applyMask(el, maskFone, reverse, apply_on_init)
    elif(mask == "cnpj"):
        jQuery(el).addClass("masked_input")
        applyMask(el, maskCNPJ, reverse, apply_on_init)
    elif(mask == "cpf"):
        jQuery(el).addClass("masked_input")
        applyMask(el, maskCPF, reverse, apply_on_init)
    elif(mask == "date"):
        jQuery(
            el
        ).off(
            'click.phanterpwaMaskdata focusin.phanterpwaMaskdata'
        ).on(
            'click.phanterpwaMaskdata focusin.phanterpwaMaskdata',
            console.info("future")
        )
        applyMask(el, maskDate, reverse, apply_on_init)
    elif(mask == "datetime"):
        jQuery(
            el
        ).off(
            'click.phanterpwaMaskdatahora focusin.phanterpwaMaskdatahora'
        ).on(
            'click.phanterpwaMaskdatahora focusin.phanterpwaMaskdatahora',
            console.info("future")
        )
        applyMask(el, maskDatetime, reverse, apply_on_init)
    elif(mask == "cep"):
        jQuery(el).addClass("masked_input")
        applyMask(el, maskCEP, reverse, apply_on_init)
    elif(mask == "real"):
        jQuery(el).addClass("masked_input")
        phanterCurrency(el,
            v_currency=currency,
            casas_decimais=casas_decimais,
            separador_decimal=separador_decimal,
            separador_milhar=separador_milhar)
    elif(mask == "decimal"):
        jQuery(el).addClass("masked_input")
        phanterDecimals(el,
            v_currency="",
            casas_decimais=casas_decimais,
            separador_decimal=separador_decimal,
            separador_milhar=separador_milhar)
    elif(mask == "off"):
        jQuery(
            el
        ).removeClass(
            "masked_input"
        ).off(
            "keypress.phanterpwaMask focusout.phanterpwaMask focusin.phanterpwaMask"
        )
    elif(mask == "custom"):
        jQuery(el).addClass("masked_input")
        applyMask(el, lambda v: baseCustom(v, parameters['mask']), reverse, apply_on_init)
    return jQuery(el)

__pragma__('nokwargs')


import re
import json
from tornado import (
    web
)


class Welcome(web.RequestHandler):
    def initialize(self, app_name, projectConfig, DALDatabase, i18nTranslator=None, logger_api=None):
        self.app_name = app_name
        self.projectConfig = projectConfig
        self.DALDatabase = DALDatabase
        self.i18nTranslator = i18nTranslator
        if logger_api:
            self.logger_api = logger_api
        if i18nTranslator:
            self.T = i18nTranslator.T
        self.phanterpwa_current_user = None

    def get(self):
        self.write({
            "status": "OK",
            "message": "Hello World!",
            "project": self.projectConfig["PROJECT"]
        })



import os
from phanterpwa.backend.request_handlers import (
    admin,
    auth,
    credentials,
    i18n_server,
    oauth
)
#  Your conttrolers
from .controllers import (
    welcome,
    developer,
)
from phanterpwa.i18n import Translator
from core import (
    projectConfig,
    logger_api
)

from .models import *

_current_dir = os.path.join(os.path.dirname(__file__))
_app_name = os.path.split(_current_dir)[-1]
_debug = projectConfig['PROJECT']['debug']
_project_name = projectConfig['PROJECT']['name']


Translator_captcha = Translator(os.path.join(_current_dir, "languages", "captcha"), identifier="{0}-captcha".format(
    _project_name), debug=_debug)
Translator_captcha.add_language("pt-BR")

Translator_api = Translator(os.path.join(_current_dir, "languages"), identifier="{0}-api".format(
    _project_name), debug=_debug)
Translator_api.add_language("pt-BR")
Translator_email = Translator(os.path.join(_current_dir, "languages", "email"), identifier="{0}-email".format(
    _project_name), debug=_debug)
Translator_email.add_language("pt-BR")

SETTINGS = {
    'debug': _debug
}


HANDLER = [
    (r"/?(api)?/?", welcome.Welcome),
    (r"/api/websocket/?", developer.EchoWebSocket, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/client/?", credentials.SignClient, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/signforms/([0-9a-zA-Z_-]+)/?", credentials.SignForms, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/signcaptchaforms/([0-9a-zA-Z_-]+)/?", credentials.SignCaptchaForms, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_captcha=Translator_captcha,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/signlockform/?", credentials.SignLockForm, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/resigncredentials/?", credentials.ReSing, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/?", auth.Auth, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_email=Translator_email,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/two-factor/([0-9a-zA-Z_\-\.]+)/?", auth.TwoFactor, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_email=Translator_email,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/lock/?", auth.LockUser, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/image/([0-9]+)/?", auth.ImageUser, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/create/?", auth.CreateAccount, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/change/?", auth.ChangeAccount, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_email=Translator_email,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/change-password/?", auth.ChangePassword, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/request-password/?", auth.RequestAccount, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_email=Translator_email,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/auth/active-account/?", auth.ActiveAccount, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        Translator_email=Translator_email,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/oauth/prompt/([a-zA-Z_-]+)?/?", oauth.Prompt, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/oauth/redirect/([a-zA-Z_-]+)?/?", oauth.Redirect, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/admin/usermanager/([0-9]+)?/?", admin.UserManager, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/i18n/([0-9a-zA-Z_-]+)/?", i18n_server.I18N, dict(
        app_name=_app_name,
        projectConfig=projectConfig
    )),
    (r"/api/projects/?([0-9a-zA-Z_-]+)?/?", developer.Projects, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/config/?([0-9a-zA-Z_-]+)?/?", developer.Configs, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),
    (r"/api/automation/([0-9a-zA-Z_-]+)/?", developer.Automation, dict(
        app_name=_app_name,
        projectConfig=projectConfig,
        DALDatabase=db,
        i18nTranslator=Translator_api,
        logger_api=logger_api
    )),

]



from org.transcrypt.stubs.browser import __pragma__
__pragma__('alias', "jQuery", "$")
__pragma__('skip')
window = jQuery = console = document = localStorage = URL =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = Date = 0
__pragma__('noskip')


__pragma__('kwargs')


class I18NServer():
    def __init__(self):
        self.storage = self.load_storage()
        self.userLang = self.get_user_lang()
        self._get_translations_on_server()

    @staticmethod
    def load_storage(self):
        result = window.PhanterPWA.CONFIG['I18N']
        return result

    def save_storage(self):
        window.PhanterPWA.CONFIG['I18N'] = self.storage

    def set_default_user_lang(self, lang):
        self.storage["default_lang"] = lang
        self.save_storage()

    @staticmethod
    def get_user_lang():
        userLang = navigator.language or navigator.userLanguage
        storage = I18NServer.load_storage()
        if "default_lang" in storage and storage['default_lang'] is not None:
            userLang = storage["default_lang"]
        return userLang

    def after_get_translations(self, data, ajax_status):
        if ajax_status == "success":
            json = data.responseJSON
            default_lang = self.storage["default_lang"]
            self.storage = json
            if default_lang is None or default_lang is js_undefined:
                self.storage["default_lang"] = default_lang
            self.save_storage()

    def _get_translations_on_server(self, new_word=None):
        if window.PhanterPWA is not js_undefined:
            if window.PhanterPWA.DEBUG and self.userLang is not None:
                window.PhanterPWA.ApiServer.GET(**{
                    'url_args': ["api", "i18n", window.PhanterPWA.get_app_name()],
                    'url_vars': {
                        "lang": self.userLang,
                        "new_word": new_word
                    },
                    'onComplete': self.after_get_translations
                })

    def get_translations(self):
        if window.PhanterPWA is not js_undefined and self.userLang is not None:
            window.PhanterPWA.ApiServer.GET(**{
                'url_args': ["api", "i18n", window.PhanterPWA.get_app_name()],
                'url_vars': {
                    "lang": self.userLang
                },
                'onComplete': self.after_get_translations
            })

    def translate(self, wordkey):
        if self.userLang in self.storage:
            if wordkey in self.storage[self.userLang]:
                return self.storage[self.userLang][wordkey]
        self._get_translations_on_server(wordkey)
        return wordkey

    def DOMTranslate(self, target):
        self.target = jQuery(target)

        def eachElement(el):
            t_elem = jQuery(el)
            sanitize = t_elem.attr("phanterpwa-i18n-sanitize")
            word = t_elem.attr("phanterpwa-i18n")
            if word is not None and word is not js_undefined:
                if t_elem[0].hasAttribute(self.userLang):
                    if sanitize is not None and sanitize is not js_undefined and str(sanitize).upper() == "FALSE":
                        t_elem.html(t_elem.attr(self.userLang))
                    else:
                        t_elem.text(t_elem.attr(self.userLang))
                else:
                    translate = self.translate(word)
                    t_elem.attr(self.userLang, translate)
                    if sanitize is not None and sanitize is not js_undefined and str(sanitize).upper() == "FALSE":
                        t_elem.html(translate)
                    else:
                        t_elem.text(translate)

        def eachTitleElement(el):
            t_elem = jQuery(el)
            word = t_elem.attr("phanterpwa-i18n-title")
            if word is not None and word is not js_undefined:
                if t_elem[0].hasAttribute(self.userLang):
                    t_elem.attr("title", t_elem.attr(self.userLang))
                else:
                    translate = self.translate(word)
                    t_elem.attr(self.userLang, translate)
                    t_elem.attr("title", translate)

        if self.target[0] is not js_undefined:
            self.target.find("span[phanterpwa-i18n]").each(lambda: eachElement(this))
            self.target.find("[phanterpwa-i18n-title]").each(lambda: eachTitleElement(this))
        return self.target


__pragma__('nokwargs')



from itsdangerous import (
    URLSafeSerializer,
    SignatureExpired,
    BadSignature
)
from datetime import (
    datetime,
    timedelta
)


class Serialize():
    def __init__(self, secret, expire_in=315576000):
        self.secret = secret
        self.expire_in = expire_in

    @property
    def expire_in(self):
        return self._expire_in

    @expire_in.setter
    def expire_in(self, value):
        self._expire_in = 315576000
        try:
            self._expire_in = int(value)
        except Exception:
            self._expire_in = 315576000

    def dumps(self, value):
        s = URLSafeSerializer(self.secret)
        new_value = {
            "expire_in": str(datetime.now() + timedelta(seconds=self.expire_in)),
            "value": value
        }
        return s.dumps(new_value)

    def loads(self, value, datetime_expire=None):
        s = URLSafeSerializer(self.secret)
        j = s.loads(value)
        v = j.get("value")
        e = j.get("expire_in")
        expire_in = datetime.fromisoformat(e)
        if datetime_expire is None:
            datetime_expire = datetime.now()
        if datetime_expire > expire_in:
            raise SignatureExpired("The token has expired")
        else:
            return v



import phanterpwa.frontend.server as server
import phanterpwa.frontend.i18n as i18n
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.components.events as events
import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.components.modal as modal
import phanterpwa.frontend.websocket as websocket
import phanterpwa.frontend.validations as validations
# from org.transcrypt.stubs.browser import __pragma__

# __pragma__('alias', "jQuery", "$")
# __pragma__('skip')

# it is ignored on transcrypt
window = jQuery = console = document = localStorage = URL = M = FormData = setTimeout = RegExp =\
    sessionStorage = this = FileReader = JSON = js_undefined = navigator = __new__ = __pragma__ = Date = 0

# __pragma__('noskip')


# __pragma__('kwargs')


XML = helpers.XML
DIV = helpers.XmlConstructor.tagger("div")
H2 = helpers.XmlConstructor.tagger("h2")
TEXTAREA = helpers.XmlConstructor.tagger("textarea")
I = helpers.XmlConstructor.tagger("i")


class PhanterPWA():
    def __init__(self, config, gates, **parameters):
        self.get_inicial_config_uri()
        self.initialize()
        if config is js_undefined or config is None:
            raise ValueError("The config is required")
        if gates is js_undefined or gates is None:
            raise ValueError("The gates is required")
        self.CONFIG = config
        self.NAME = config.PROJECT.name
        self.TITLE = config.PROJECT.title
        self.VERSION = config.PROJECT.version
        self.COMPILATION = config.PROJECT.compilation
        self.AUTHOR = config.PROJECT.author
        self.DEBUG = config.PROJECT.debug
        self.Gates = dict(gates)
        self.ProgressBar = progressbar.ProgressBar("#main-progress-bar-container")
        self.Request = WayRequest()
        self.Components = {}
        self.Events = {}
        self.Cache = {}
        self.Response = None
        self.default_way = parameters.get("default_way", "home")
        self._after_open_way = parameters.get("after_open_way", None)
        self.counter = 0
        self.states = dict()
        self._social_login_icons = {
            "google": I(_class="fab fa-google"),
            "facebook": I(_class="fab fa-facebook"),
            "twitter": I(_class="fab fa-twitter")
        }

        # jQuery(document).ajaxComplete(
        #     lambda event, xhr, options: self._after_ajax_complete(event, xhr, options)
        # )
        window.PhanterPWA = self
        if self.DEBUG:
            console.info("starting {0} application (version: {1}, compilation: {2})".format(
                self.CONFIG.PROJECT.title, self.CONFIG.PROJECT.version, self.CONFIG.PROJECT.compilation)
            )
        self.add_event(events.Waves())
        self.add_event(events.WidgetsInput())
        self.add_event(events.WayHiperlinks())
        self.ApiServer = server.ApiServer()
        self.ApiServer.getClientToken()
        self.I18N = i18n.I18NServer()
        self.WS = websocket.WebSocketPhanterPWA(self.CONFIG["APP"]["http_address"])
        if self.DEBUG:
            self.add_component(Developer_Toolbar())
        self.Valider = validations.Valid

    @staticmethod
    def check_event_namespace(el, event_name, namespace):
        element = jQuery(el)
        if element.length > 0:
            if element.length == 1:
                obj_events = jQuery._data(element[0], 'events')
                if obj_events is not js_undefined:
                    event_list = obj_events[event_name]
                    if isinstance(event_list, list):
                        for x in event_list:
                            ns = x.namespace
                            if ns is not js_undefined and ns == namespace:
                                return True
            else:
                console.error("check_event_namespace must be used in jquery selectors with",
                    "an element like when using the id selector")
        return False

    @staticmethod
    def ibind(el, event_name, namespace, callback):
        if not window.PhanterPWA.check_event_namespace(el, event_name, namespace) and callable(callback):
            ns = "{0}.{1}".format(event_name, namespace)
            jQuery(el).off(ns).on(ns, lambda event: callback(this, event))

    def get_inicial_config_uri(self):
        initial_config = __new__(URL(window.location.href))
        params = initial_config.searchParams
        authorization = params.js_get("authorization")
        client_token = params.js_get("client_token")
        url_token = params.js_get("url_token")
        auth_user = params.js_get("auth_user")
        redirect = params.js_get("redirect")
        way = params.js_get('way')
        if auth_user is not None and auth_user is not js_undefined:
            auth_user = JSON.parse(auth_user)

        if (authorization is not None) and (url_token is not None) and\
                (auth_user is not None) and (client_token is not None):
            localStorage.setItem('phanterpwa-client-token', client_token)
            localStorage.setItem('phanterpwa-url-token', url_token)
            if auth_user["remember_me"] is True:
                localStorage.setItem("phanterpwa-authorization", authorization)
                localStorage.setItem("auth_user", JSON.stringify(auth_user))
                sessionStorage.removeItem("phanterpwa-authorization")
                sessionStorage.removeItem("auth_user")
            else:
                sessionStorage.setItem("phanterpwa-authorization", authorization)
                sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
                localStorage.removeItem("phanterpwa-authorization")
                localStorage.removeItem("auth_user")
            localStorage.setItem("last_auth_user", JSON.stringify(auth_user))
        if redirect is not None and redirect is not js_undefined:
            window.location = redirect
        if way is not None and way is not js_undefined:
            self.open_way(way)

    def _after_ajax_complete(self, event, xhr, option):
        if option is not js_undefined:
            p_url = self.parse_url(option.url)
            self.reload_components(ajax=p_url)
            self.reload_events(ajax=p_url)

    def get_id(self, salt="phanterpwa"):
        self.counter += 1
        timestamp = __new__(Date().getTime())
        if salt is None or salt is js_undefined:
            console.error("The salt of method get_id is invalid! given:", salt)
            salt = "phanterpwa"
        return "{0}-{1}-{2}".format(salt, self.counter, timestamp)

    def social_login_list(self):
        social_logins = window.PhanterPWA["CONFIG"]["SOCIAL_LOGINS"]
        s_logins = dict()
        if social_logins is not None and social_logins is not js_undefined:
            s_logins = dict(social_logins)
        list_login = social_logins.keys()
        l = []
        for x in list_login:
            if x in self._social_login_icons:
                l.append([x, self._social_login_icons[x]])
            else:
                l.append([x, I(_class="fas fa-at")])
        return l

    @staticmethod
    def get_app_name(self):
        return window.PhanterPWA.CONFIG["APP"]["name"]

    @staticmethod
    def get_api_address(self):
        return window.PhanterPWA.CONFIG["APP"]["http_address"]

    @staticmethod
    def flash(msg=None, **parameters):
        timeout = parameters.get("timeout", 3000)
        timestamp = __new__(Date().getTime())
        msg_id = "phanterpwa-flash-wrapper-{0}".format(timestamp)
        target = jQuery("#phanterpwa-flash-container")
        if target.length == 0:
            jQuery("body").prepend(DIV(_id="phanterpwa-flash-container").jquery())

        def remove_msg():
            jQuery("#{0}".format(msg_id)).removeClass("enabled").slideUp(500)
            setTimeout(lambda: jQuery("#{0}".format(msg_id)).remove(), 2000)
        if "html" in parameters:
            msg = parameters.get("html", None)
            if msg is not None:
                html = DIV(
                    XML(msg),
                    _id=msg_id,
                    _class="phanterpwa-flash-wrapper enabled",
                    _style="opacity: 0; margin-top:-20px"
                )
                html.append_to("#phanterpwa-flash-container")
                jQuery("#{0}".format(msg_id)).addClass("enabled").animate({'opacity': 1, 'margin-top': '0px'})
                setTimeout(remove_msg, timeout)
        elif msg is not None and msg is not js_undefined:
            html = DIV(
                msg,
                _id=msg_id,
                _class="phanterpwa-flash-wrapper",
                _style="opacity: 0; margin-top:-20px"
            )
            html.append_to("#phanterpwa-flash-container").find(
                "#{0}".format(msg_id)).addClass("enabled").animate({'opacity': 1, 'margin-top': '0px'})
            setTimeout(remove_msg, timeout)

    def reload(self, **context):
        self.reload_components(**context)
        self.reload_events(**context)

    def reload_components(self, **context):
        for c in self.Components.keys():
            if callable(self.Components[c].reload):
                if self.Components[c].actived is False:
                    if self.DEBUG:
                        console.info("Reload Components {0}".format(c))
                    self.Components[c].actived = True
                    self.Components[c].reload(**context)

    def reload_events(self, **context):
        for c in self.Events.keys():
            if callable(self.Events[c].reload):

                self.Events[c].reload(**context)
        target = context.get("selector", None)
        ajax = context.get("ajax", None)
        if target is not None:
            self.I18N.DOMTranslate(target)
        elif ajax is not None and len(ajax[1]) > 0 and ajax[1][0] == "signcaptchaforms":
            self.I18N.DOMTranslate(".phanterpwa-widget-captcha-container")

    def async_function(self, callback, delay=0, **context):
        def f():
            if self.DEBUG:
                console.info("Async Callback", callback)
            callback()
            self.reload_components(**context)
            self.reload_events(**context)

        if callable(callback):
            setTimeout(f, delay)

    def add_component(self, component):
        if isinstance(component, Component):
            self.Components[component.identifier] = component
            component.start()
        elif isinstance(component, list):
            for x in component:
                self.add_component(x)
        else:
            console.error("The component must be Component instance")

    def add_event(self, event):
        if isinstance(event, events.Event):
            self.Events[event.identifier] = event
            event.start()
        elif isinstance(event, list):
            for x in event:
                self.add_event(x)
        else:
            console.error("The event must be Event instance")

    def remove_last_auth_user(self):
        localStorage.removeItem("last_auth_user")

    def _after_submit_login(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if ajax_status == "success":
            if data.status == 200:
                authorization = json.authorization
                auth_user = json.auth_user
                client_token = json.client_token
                url_token = json.url_token
                if (authorization is not js_undefined) and\
                        (auth_user is not js_undefined) and (client_token is not js_undefined):
                    localStorage.setItem('phanterpwa-client-token', client_token)
                    localStorage.setItem('phanterpwa-url-token', url_token)
                    if auth_user["remember_me"] is True:
                        localStorage.setItem("phanterpwa-authorization", authorization)
                        localStorage.setItem("auth_user", JSON.stringify(auth_user))
                        sessionStorage.removeItem("phanterpwa-authorization")
                        sessionStorage.removeItem("auth_user")
                    else:
                        sessionStorage.setItem("phanterpwa-authorization", authorization)
                        sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
                        localStorage.removeItem("phanterpwa-authorization")
                        localStorage.removeItem("auth_user")
                    self.WS.send("command_online")
                    localStorage.setItem("last_auth_user", JSON.stringify(auth_user))
                window.PhanterPWA.open_current_way()
            elif data.status == 206:
                client_token = json.client_token
                if client_token is not js_undefined:
                    localStorage.setItem('phanterpwa-client-token', client_token)
                    sessionStorage.removeItem("phanterpwa-authorization")
                    sessionStorage.removeItem("auth_user")
                    localStorage.removeItem("phanterpwa-authorization")
                    localStorage.removeItem("auth_user")
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callable(callback):
            callback(data, ajax_status)

    def login(self, csrf_token, username, password, remember_me, **parameters):
        if remember_me is None or remember_me is js_undefined:
            remember_me = False
        callback = parameters.get("callback", None)
        formdata = __new__(FormData())
        formdata.append(
            "csrf_token",
            csrf_token
        )
        login_password = "{0}:{1}".format(
            window.btoa(username),
            window.btoa(password)
        )
        formdata.append("edata", login_password)
        formdata.append(
            "remember_me",
            remember_me
        )
        window.PhanterPWA.ApiServer.POST(**{
            'url_args': ["api", "auth"],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_login(
                data, ajax_status, callback)
        })

    def _after_submit_two_factor(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if ajax_status == "success":
            if data.status == 200:
                authorization = json.authorization
                auth_user = json.auth_user
                client_token = json.client_token
                url_token = json.url_token
                if (authorization is not js_undefined) and\
                        (auth_user is not js_undefined) and (client_token is not js_undefined):
                    localStorage.setItem('phanterpwa-client-token', client_token)
                    localStorage.setItem('phanterpwa-url-token', url_token)
                    if auth_user["remember_me"] is True:
                        localStorage.setItem("phanterpwa-authorization", authorization)
                        localStorage.setItem("auth_user", JSON.stringify(auth_user))
                        sessionStorage.removeItem("phanterpwa-authorization")
                        sessionStorage.removeItem("auth_user")
                    else:
                        sessionStorage.setItem("phanterpwa-authorization", authorization)
                        sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
                        localStorage.removeItem("phanterpwa-authorization")
                        localStorage.removeItem("auth_user")
                    self.WS.send("command_online")
                    localStorage.setItem("last_auth_user", JSON.stringify(auth_user))
                window.PhanterPWA.open_current_way()
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callable(callback):
            callback(data, ajax_status)

    def two_factor(self, url_authorization, code, **parameters):
        callback = parameters.get("callback", None)
        formdata = __new__(FormData())
        formdata.append(
            "code",
            code
        )
        window.PhanterPWA.ApiServer.PUT(**{
            'url_args': ["api", "auth", "two-factor", url_authorization],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_two_factor(
                data, ajax_status, callback)
        })

    def social_login(self, social_name, callback=None):
        window.PhanterPWA.ApiServer.GET(**{
            'url_args': ['api', 'oauth', 'prompt', social_name],
            'onComplete': lambda data, ajax_status: self._after_get_social_login(
                data, ajax_status, callback
            )
        })

    def _after_get_social_login(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if ajax_status == "success":
            window.location = json.redirect
        if callable(callback):
            callback(data, ajax_status)

    def _after_submit_register(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if ajax_status == "success":
            authorization = json.authorization
            auth_user = json.auth_user
            client_token = json.client_token
            url_token = json.url_token
            if (authorization is not js_undefined) and\
                    (auth_user is not js_undefined) and (client_token is not js_undefined):
                localStorage.setItem('phanterpwa-client-token', client_token)
                localStorage.setItem('phanterpwa-url-token', url_token)
                if auth_user["remember_me"] is True:
                    localStorage.setItem("phanterpwa-authorization", authorization)
                    localStorage.setItem("auth_user", JSON.stringify(auth_user))
                    sessionStorage.removeItem("phanterpwa-authorization")
                    sessionStorage.removeItem("auth_user")
                else:
                    sessionStorage.setItem("phanterpwa-authorization", authorization)
                    sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
                    localStorage.removeItem("phanterpwa-authorization")
                    localStorage.removeItem("auth_user")
                self.WS.send("command_online")
                localStorage.setItem("last_auth_user", JSON.stringify(auth_user))
        else:
            console.info(data.status)
        if self.DEBUG:
            console.info(json.i18n.message)
        if callback is not None:
            callback(data, ajax_status)

    def register(self, csrf_token, first_name, last_name, email, password, password_repeat, **parameters):
        callback = None
        if "callback" in parameters:
            callback = parameters["callback"]
        formdata = __new__(FormData())
        formdata.append(
            "csrf_token",
            csrf_token
        )
        formdata.append(
            "first_name",
            first_name
        )
        formdata.append(
            "last_name",
            last_name
        )
        formdata.append(
            "email",
            email
        )
        passwords = "{0}:{1}".format(
            window.btoa(password),
            window.btoa(password_repeat)
        )
        formdata.append(
            "edata",
            passwords
        )
        window.PhanterPWA.ApiServer.POST(**{
            'url_args': ["api", "auth", "create"],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_register(
                data, ajax_status, callback)
        })

    def _after_submit_request_password(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callback is not None:
            callback(data, ajax_status)

    def request_password(self, csrf_token, email, **parameters):
        last_auth_user = self.get_last_auth_user()
        if email != last_auth_user.email:
            self.remove_last_auth_user()
        callback = None
        if "callback" in parameters:
            callback = parameters["callback"]
        formdata = __new__(FormData())
        formdata.append(
            "csrf_token",
            csrf_token
        )
        formdata.append(
            "email",
            email
        )
        window.PhanterPWA.ApiServer.POST(**{
            'url_args': ["api", "auth", "request-password"],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_request_password(
                data, ajax_status, callback)
        })

    def _after_submit_activation_account(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if ajax_status == "success":
            auth_user = json.auth_user
            sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
            localStorage.setItem("last_auth_user", JSON.stringify(auth_user))
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callback is not None:
            callback(data, ajax_status)

    def activation_account(self, csrf_token, activation_code, **parameters):
        callback = None
        if "callback" in parameters:
            callback = parameters["callback"]
        formdata = __new__(FormData())
        formdata.append(
            "csrf_token",
            csrf_token
        )
        formdata.append(
            "activation_code",
            activation_code
        )
        window.PhanterPWA.ApiServer.POST(**{
            'url_args': ["api", "auth", "active-account"],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_activation_account(
                data, ajax_status, callback)
        })

    def _after_submit_change_password(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callback is not None:
            callback(data, ajax_status)

    def change_password(self, csrf_token, password, new_password, new_password_repeat, **parameters):
        callback = None
        if "callback" in parameters:
            callback = parameters["callback"]
        formdata = __new__(FormData())
        formdata.append(
            "csrf_token",
            csrf_token
        )
        formdata.append(
            "edata",
            "{0}:{1}:{2}".format(
                window.btoa(password),
                window.btoa(new_password),
                window.btoa(new_password_repeat),
            )
        )
        window.PhanterPWA.ApiServer.POST(**{
            'url_args': ["api", "auth", "change-password"],
            'form_data': formdata,
            'onComplete': lambda data, ajax_status: self._after_submit_change_password(
                data, ajax_status, callback)
        })

    def logout(self, **parameters):
        callback = None
        if "callback" in parameters:
            callback = parameters["callback"]
        self.WS.send("command_offline")
        sessionStorage.removeItem("phanterpwa-authorization")
        sessionStorage.removeItem("auth_user")
        localStorage.removeItem("phanterpwa-authorization")
        localStorage.removeItem("auth_user")
        self.open_default_way()
        if callback is not None:
            callback()

    def _after_get_csrf_token(self, data, ajax_status, callback=None):
        json = data.responseJSON
        if self.DEBUG:
            console.info(data.status, json.i18n.message)
        if callback is not None:
            callback(data, ajax_status)

    def get_csrf_token(self, table_name, **parameters):
        callback = None
        headers = None
        if "callback" in parameters:
            callback = parameters["callback"]
        if "headers" in parameters:
            headers = parameters["headers"]

        window.PhanterPWA.ApiServer.GET(**{
            "url_args": ["api", "signforms", table_name],
            "onComplete": lambda data, ajax_status: self._after_get_csrf_token(
                data, ajax_status, callback),
            "headers": headers
        })

    @staticmethod
    def xml_to_dom_element(xml, target, callback=None):
        target = jQuery(target)
        if target.length == 0:
            console.error("The target element don't exists!")
        if isinstance(xml, helpers.XmlConstructor):
            target.html(xml.jquery())
        else:
            target.html(xml)
        if callable(callback):
            callback(target)
        window.PhanterPWA.I18N.DOMTranslate(target)

    @staticmethod
    def get_current_way():
        current_way = sessionStorage.getItem("current_way")
        if current_way is None or current_way is js_undefined or current_way is "lock":
            current_way = "home"
        return current_way

    @staticmethod
    def open_code_way(code=500, request=None, response=None):
        if str(code).isdigit():
            code = int(code)
        if code not in window.PhanterPWA.Gates:
            if window.PhanterPWA.DEBUG:
                console.info(code, request, response)
            if code == 401:
                gatehandler.Error_401(request, response)
            elif code == 403:
                gatehandler.Error_403(request, response)
            elif code == 404:
                gatehandler.Error_404(request, response)
            else:
                gatehandler.Error_502(request, response)
        else:
            if isinstance(request, WayRequest):
                window.PhanterPWA.Gates[code](request, response)
            else:
                if window.PhanterPWA.DEBUG:
                    console.error("The request must be WayRequest instance.")
                gatehandler.Error_500(request, response)

    @staticmethod
    def get_client_token():
        client_token = localStorage.getItem("phanterpwa-client-token")
        if client_token is not None and client_token is not js_undefined:
            return client_token
        else:
            return None

    @staticmethod
    def get_url_token():
        url_token = localStorage.getItem("phanterpwa-url-token")
        if url_token is not None and url_token is not js_undefined:
            return url_token
        else:
            return None

    @staticmethod
    def get_authorization():
        authorization = sessionStorage.getItem("phanterpwa-authorization")
        if authorization is None or authorization is js_undefined:
            authorization = localStorage.getItem("phanterpwa-authorization")
        else:
            localStorage.removeItem("phanterpwa-authorization")
        if authorization is not None and authorization is not js_undefined:
            return authorization
        else:
            localStorage.removeItem("auth_user")
            sessionStorage.removeItem("auth_user")
            return None

    @staticmethod
    def get_auth_user():
        auth_user = sessionStorage.getItem("auth_user")
        if auth_user is None or auth_user is js_undefined:
            auth_user = localStorage.getItem("auth_user")
        else:
            localStorage.removeItem("auth_user")
        if auth_user is not None and auth_user is not js_undefined:
            return JSON.parse(auth_user)
        else:
            window.PhanterPWA.WS.send("command_offline")
            localStorage.removeItem("phanterpwa-authorization")
            sessionStorage.removeItem("phanterpwa-authorization")
            return None

    @staticmethod
    def store_auth_user(auth_user):
        if auth_user is not None and auth_user is not js_undefined:
            if auth_user["remember_me"] is True:
                localStorage.setItem("auth_user", JSON.stringify(auth_user))
                sessionStorage.removeItem("auth_user")
            else:
                sessionStorage.setItem("auth_user", JSON.stringify(auth_user))
                localStorage.removeItem("auth_user")
            localStorage.setItem("last_auth_user", JSON.stringify(auth_user))

    @staticmethod
    def get_auth_user_image():
        image_user = "/static/{0}/images/user.png".format(
            window.PhanterPWA.VERSIONING)
        auth_user = window.PhanterPWA.get_auth_user()
        if auth_user is not None and auth_user['image'] is not None:
            url_token = localStorage.getItem("phanterpwa-url-token")
            server = window.PhanterPWA.CONFIG["APP"]["http_address"]
            image_user = "{0}/api/auth/image/{1}?sign={2}".format(
                server,
                auth_user['image'],
                url_token
            )
        return image_user

    @staticmethod
    def get_last_auth_user_image():
        image_user = "/static/{0}/images/user.png".format(
            window.PhanterPWA.VERSIONING)
        auth_user = window.PhanterPWA.get_last_auth_user()
        if auth_user is not None and auth_user['image'] is not None:
            url_token = localStorage.getItem("phanterpwa-url-token")
            server = window.PhanterPWA.CONFIG["APP"]["http_address"]
            image_user = "{0}/api/auth/image/{1}?sign={2}".format(
                server,
                auth_user['image'],
                url_token
            )
        return image_user

    @staticmethod
    def get_last_auth_user():
        last_auth_user = localStorage.getItem("last_auth_user")
        if last_auth_user is not None and last_auth_user is not js_undefined:
            return JSON.parse(last_auth_user)
        else:
            return None

    def open_way(self, way):
        self.Request = WayRequest()
        self.Request.open_way(way)
        self._after_ajax_complete()
        if self._after_open_way is not None and self._after_open_way is not js_undefined:
            self._after_open_way(self.Request)

    def open_current_way(self):
        self.open_way(self.get_current_way())

    def open_default_way(self):
        self.open_way(self.default_way)

    def initialize(self):
        if self.DEBUG:
            console.info("initializing...")

    def parse_url(self, url):
        l = __new__(URL(url))
        t_args = l.pathname.split("/")[1:]
        gate = t_args[0]
        c_args = t_args[1:]
        n_args = list()
        for c in c_args:
            if c is not "":
                n_args.append(c)

        params = dict()
        if l.search is not "":

            def add_in_p(k, v):
                params[k] = v
            t_params = l.searchParams
            t_params.forEach(lambda v, k: add_in_p(k, v))
        return [gate, n_args, params, url]

    def parse_way(self, way):
        gate = way
        if "?" in way or "/" in way:
            url = "{0}/{1}".format(window.PhanterPWA.CONFIG.CONFIGJS.api_server_address, way)
            return self.parse_url(url)
        else:
            return {gate, [], {}, way}

    def GET(self, **parameters):
        self.ApiServer.GET(**parameters)

    def DELETE(self, **parameters):
        self.ApiServer.DEL(**parameters)

    def POST(self, **parameters):
        self.ApiServer.POST(**parameters)

    def PUT(self, **parameters):
        self.ApiServer.PUT(**parameters)

    def LOAD(self, **parameters):
        Loads(**parameters)


class Loads():
    def __init__(self, **parameters):
        self.url_args = parameters.get("args", None)
        self.url_vars = parameters.get("vars", {})
        self.onComplete = parameters.get("onComplete", None)
        pro_args = self._process_args()
        location = __new__(URL(window.location))
        origin = location.origin
        url = "{0}/{1}".format(origin, pro_args)
        jQuery.js_get(url, self._after_load)

    def _after_load(self, data, ajax_status):
        context = self._process_vars()
        for x in context.keys():
            k = "".join(["{{", x, "}}"])
            ls = data.split(k)
            ns = str(context[x]).join(ls)
            data = ns
        if callable(self.onComplete):
            self.onComplete(data)
            window.PhanterPWA.reload()

    def _process_args(self):
        s_args = ""
        if isinstance(self.url_args, list):
            s_args = "/".join(self.url_args)
        else:
            console.error("LOAD url_args must be list(array).")
        if s_args != "":
            s_args = "{0}".format(s_args)
        return s_args

    def _process_vars(self):
        try:
            _vars = dict(self.url_vars)
        except Exception as e:
            console.error("LOAD url_vars must be dict object")
        if isinstance(_vars, dict):
            return _vars
        else:
            return {}


class Component(helpers.XmlConstructor):
    def __init__(self, identifier, *content, **attributes):
        self.actived = False
        self.identifier = identifier
        self._identifier = window.PhanterPWA.get_id(identifier)
        attributes["_phanterpwa-component"] = self._identifier
        attributes["_id"] = identifier
        if len(content) == 0:
            content = [DIV("xml content component", _style="color: red;")]
        helpers.XmlConstructor.__init__(self, "phanterpwa-component", False, *content, **attributes)
        window.PhanterPWA.add_component(self)

    def _onload(self):
        self.start()

    def reload(self, **context):
        if window.PhanterPWA.DEBUG:
            console.info("The Component {0} reload with context {1}".format(self.identifier, context))

    def start(self):
        if window.PhanterPWA.DEBUG:
            console.info("The Component {0} starts".format(self.identifier))


class Developer_Toolbar(Component):
    def __init__(self):
        Component.__init__(self, "developer_toolbar")

    def _switch_panel_objects(self):
        t = jQuery("#p-developer_toolbar")
        if t.hasClass("enabled"):
            t.removeClass("enabled")
        else:
            t.addClass("enabled")

    def _get_beautify(self):
        seen = []

        def f(k, v):
            if v is not None and __pragma__("js", "{}", "typeof v == 'object'"):
                test = __pragma__("js", "{}", "seen.indexOf(v) >= 0")
                if test:
                    return
                seen.append(v)
            return v
        response = JSON.stringify(window.PhanterPWA.Response, lambda k, v: f(k, v))
        n = JSON.parse(response)
        if n is not None and n is not js_undefined:
            request = JSON.stringify(n._request, None, 4)
            del n._request
            response = JSON.stringify(n, None, 4)
        else:
            response = None
            request = None

        return [response, request]

    def _open_modal(self):
        rr = self._get_beautify()
        request = TEXTAREA(rr[1])
        response = TEXTAREA(rr[0])
        content_ = DIV(
            DIV(
                H2("Request"),
                request,
                _class='phanterpwa-developer_toolbar phanterpwa-developer_toolbar-request'
            ),
            DIV(
                H2("Response"),
                response,
                _class='phanterpwa-developer_toolbar phanterpwa-developer_toolbar-response'
            ),
            _class="phanterpwa-developer_toolbar-objects"
        )
        self.m_modal = modal.Modal(
            "#p-developer_toolbar",
            **{
                "title": "Developer Toolbar",
                "content": content_
            }
        )
        self.m_modal.open()

    def check_duplicates_ids(self):
        def check(el):
            ids = jQuery("[id='{0}']".format(el.id))
            if ids.length > 1 and ids[0] == el:
                console.warn('Developer_Toolbar: Multiple IDs #{0}'.format(el.id))
        jQuery('[id]').each(lambda: check(this))

    def _binds(self):
        t = jQuery("#p-developer_toolbar")
        t.find(".button_developer").off("click.developer_toolbar_button").on(
            "click.developer_toolbar_button",
            self._open_modal
        )
        jQuery(window).resize(lambda: self._window_size())
        self.check_duplicates_ids()
        self._window_size()

    def _window_size(self):
        w = jQuery(window).width()
        h = jQuery(window).height()
        jQuery("#p-developer_toolbar").find(".current-size").html("{0}x{1} - ".format(w, h))

    def start(self):
        if jQuery("#p-developer_toolbar").length == 0:
            xml = DIV(
                _id="p-developer_toolbar"
            )
            xml.append_to(
                "body"
            )
        self._binds()


class WayRequest():
    def __init__(self):
        self.timestamp = __new__(Date().getTime())
        self.application_info = None
        self.auth_user = None
        self.gate = "home"
        self.way = "home"
        self.params = None
        self.args = None
        self.last_way = "home"
        self._element = None
        self.widgets = {}
        self.error = None

    @staticmethod
    def _body_flag(*new_flag):
        body = jQuery("body")
        lclass = body.attr("class")
        r = __new__(RegExp(r"/\s+/"))
        if lclass is not js_undefined:
            lclass = lclass.split(r)
            for x in lclass:
                if x.startswith("phanterpwa-flag-"):
                    body.removeClass(x)
        for f in new_flag:
            if f.startswith("phanterpwa-flag-"):
                f = f.replace("/", "_")
                body.addClass(f)
            else:
                f = f.replace("/", "_")
                body.addClass("phanterpwa-flag-{0}".format(f))

    def add_widget(self, widget):
        if isinstance(widget, widgets.Widget):
            self.widgets[widget.identifier] = widget
            if callable(widget.start):
                widget.start()

    def _process_way(self, way):
        self.application_info = "{0} (version: {1}, compilation: {2})".format(
            window.PhanterPWA.CONFIG.PROJECT.title,
            window.PhanterPWA.CONFIG.PROJECT.version,
            window.PhanterPWA.CONFIG.PROJECT.compilation
        )
        self.auth_user = window.PhanterPWA.get_auth_user()
        self.gate = way
        self.way = way
        if "?" in self.way or "/" in self.way:
            l = "{0}/{1}".format(window.PhanterPWA.CONFIG.CONFIGJS.api_server_address, self.way)
            l = __new__(URL(l))
            t_args = l.pathname.split("/")[1:]
            self.gate = t_args[0]
            c_args = t_args[1:]
            n_args = list()
            for c in c_args:
                if c is not "":
                    n_args.append(c)

            if len(n_args) > 0:
                self.args = n_args

            if l.search is not "":
                p = dict()

                def add_in_p(k, v):
                    p[k] = v
                t_params = l.searchParams
                t_params.forEach(lambda v, k: add_in_p(k, v))
                self.params = p

    def get_arg(self, arg):
        if arg is not None and arg is not js_undefined:
            if len(self.args) >= int(arg) + 1:
                return self.args[arg]
            else:
                return None
        else:
            return None

    def get_param(self, k):
        return self.params.get(k, None)

    def _open_way(self, way):
        self._process_way(way)
        self._body_flag(self.gate, way)
        last_way = window.PhanterPWA.get_current_way()
        self.last_way = last_way
        if self.gate in window.PhanterPWA.Gates:
            sessionStorage.setItem("current_way", self.way)
            try:
                window.PhanterPWA.Gates[self.gate](self)
            except Exception:
                console.error("Error on try open '{0}'".format(way))
                window.PhanterPWA.Gates[404](self)
        else:
            self.error = 404
            window.PhanterPWA.Gates[404](self)
        if self.DEBUG:
            if self._element is not None:
                console.info(
                    "Using the element ", self._element, "to try way '{0}'. ".format(self.way), "request: ", self)
            else:
                console.info("Try programatically way to '{0}'. ".format(self.way), "request: ", self)
        setTimeout(
            lambda: window.PhanterPWA.ProgressBar.removeEventProgressBar("OPEN_WAY_{0}".format(self.timestamp)), 300)

    def open_way(self, way):
        self.timestamp = __new__(Date().getTime())
        window.PhanterPWA.ProgressBar.addEventProgressBar("OPEN_WAY_{0}".format(self.timestamp))
        setTimeout(lambda: self._open_way(way), 30)


__pragma__('nokwargs')




from ..helpers import DIV, I, IMG
from ..xmlconstructor import XmlConstructor


class LeftBarSubMenu(XmlConstructor):
    def __init__(self, _id, label, **attributes):
        self._id = _id
        self.label = label
        self.initial_class = "phanterpwa-component-left_bar-submenu-button link"
        attributes["_id"] = "phanterpwa-component-left_bar-submenu-button-%s" % _id
        if "_class" in attributes:
            self.initial_class = " ".join(
                [attributes['_class'].strip(), "phanterpwa-component-left_bar-submenu-button link"])
        attributes['_class'] = self.initial_class
        content = [
            DIV(I(_class="fas fa-angle-right"), _class="phanterpwa-component-left_bar-submenu-icon-container"),
            DIV(self.label, _class="phanterpwa-component-left_bar-submenu-label"),
        ]
        XmlConstructor.__init__(self, 'div', False, *content, **attributes)


class LeftBarMenu(XmlConstructor):
    def __init__(self, _id, label, icon_class, **attributes):
        self._id = _id
        self.label = label
        self.icon_class = icon_class
        self.submenus = []
        self.componentSubmenu = LeftBarSubMenu
        initial_class = "phanterpwa-component-left_bar"
        initial_id = "phanterpwa-component-left_bar-%s" % _id
        self.button_attributes = attributes
        if "_class" in self.button_attributes:
            self.button_attributes["_class"] = " ".join([
                self.button_attributes['_class'].strip(),
                "phanterpwa-component-left_bar-menu link"])
        else:
            self.button_attributes["_class"] = "phanterpwa-component-left_bar-menu link"
        XmlConstructor.__init__(self, 'div', False, _id=initial_id, _class=initial_class)
        self._update_content()

    def addSubmenu(self, _id, label, **attributes):
        self.submenus.append(self.componentSubmenu(_id=_id, label=label, **attributes))
        self._update_content()

    def _update_content(self):
        html_submenus = ""
        attributes = self.button_attributes
        if self.submenus:
            attributes["_target_submenu"] = "phanterpwa-component-left_bar-submenu-from-%s" % self._id
            html_submenus = DIV(
                *self.submenus,
                _id=attributes["_target_submenu"],
                _class="phanterpwa-component-left_bar-submenu-container")
        self.content = [
            DIV(
                DIV(I(_class=self.icon_class),
                    _class="phanterpwa-component-left_bar-icon-container"),
                DIV(self.label, _class="phanterpwa-component-left_bar-label"),
                **attributes),
            html_submenus
        ]


class LeftBarButton(XmlConstructor):
    def __init__(self, _id, label, icon_class, **attributes):
        self._id = _id
        self.label = label
        self.icon_class = icon_class
        self.submenus = []
        initial_class = "phanterpwa-component-left_bar"
        initial_id = "phanterpwa-component-left_bar-%s" % _id
        self.button_attributes = attributes
        if "_class" in self.button_attributes:
            self.button_attributes["_class"] = " ".join([
                self.button_attributes['_class'].strip(),
                "phanterpwa-component-left_bar-button link"])
        else:
            self.button_attributes["_class"] = "phanterpwa-component-left_bar-button link"
        XmlConstructor.__init__(self, 'div', False, _id=initial_id, _class=initial_class)
        self.content = [
            DIV(
                DIV(I(_class=self.icon_class),
                    _class="phanterpwa-component-left_bar-icon-container"),
                DIV(self.label, _class="phanterpwa-component-left_bar-label"),
                **self.button_attributes)
        ]


class LeftBarUserMenu(XmlConstructor):
    def __init__(self, _id, name_user="Nome usuário", url_image_user="/static/images/user.png", **attributes):
        self._id = _id
        self.name_user = name_user
        self.url_image_user = url_image_user
        self.submenus = []
        self._image = IMG(_id="phanterpwa-component-left_bar-url-imagem-user",
            _src=url_image_user,
            _alt="user avatar")
        self.componentSubmenu = LeftBarSubMenu
        initial_class = "phanterpwa-component-left_bar"
        initial_id = "phanterpwa-component-left_bar-%s" % _id
        self.button_attributes = attributes
        if "_class" in self.button_attributes:
            self.button_attributes["_class"] = " ".join([
                self.button_attributes['_class'].strip(),
                "phanterpwa-component-left_bar-button-user cmp-bar-user-img link"])
        else:
            self.button_attributes["_class"] = "phanterpwa-component-left_bar-menu link"
        XmlConstructor.__init__(self, 'div', False, _id=initial_id, _class=initial_class)
        self._update_content()

    def addSubmenu(self, _id, label, **attributes):
        self.submenus.append(self.componentSubmenu(_id=_id, label=label, **attributes))
        self._update_content()

    def _update_content(self):
        html_submenus = ""
        attributes = self.button_attributes
        if self.submenus:
            attributes["_target_submenu"] = "phanterpwa-component-left_bar-submenu-from-%s" % self._id
            html_submenus = DIV(
                *self.submenus,
                _id=attributes["_target_submenu"],
                _class="phanterpwa-component-left_bar-submenu-container")
        self.content = [
            DIV(
                DIV(
                    DIV(self._image,
                        _class="phanterpwa-component-left_bar-image-user"),
                    _class="phanterpwa-component-left_bar-image-user-container"),
                DIV(self.name_user,
                    _id="phanterpwa-component-left_bar-name-user",
                    _class="phanterpwa-component-left_bar-label"),
                **attributes),
            html_submenus
        ]


# -*- coding: utf-8 -*-
import random
import os
import json
from glob import glob
from phanterpwa.helpers import DIV, XML, SVG, SPAN
from phanterpwa.i18n import Translator
from phanterpwa.backend.security import (
    Serialize,
    SignatureExpired,
    BadSignature,
    URLSafeSerializer,
)
__dirname__ = os.path.dirname(__file__)


class Captcha(object):
    def __init__(self,
        _id,
        secret_key,
        time_token_expire,
        num_opt=4,
        question="Which figure below corresponds to: {option}.",
        debug=False,
        translator=None):
        super(Captcha, self).__init__()
        self.debug = debug
        self.secret_key = secret_key
        self.time_token_expire = time_token_expire
        self.serializer = Serialize(
            self.secret_key,
            self.time_token_expire
        )

        self._background_color = {
            "red": "#990000",
            "yellow": "#FFCC00",
            "blue": "#6699FF"
        }
        self._foreground_color = {
            "black": "#1F1A17",
            "white": "white",
            "green": "#009933"
        }

        self._signature = None
        self._choice = None
        self.translator = translator
        self.color_pairs = dict()
        self.options = dict()
        self._opt = []
        self.recipes = dict()
        self.classification = set()
        self.vectors = dict()
        self.grafical_forms = dict()
        self.svg_forms = dict()
        self.question = question
        if isinstance(_id, str):
            self._id = _id
        else:
            raise ValueError("The captcha id must be string. Given: {0}".format(type(_id)))
        self.num_opt = num_opt
        self.keys_attributes = []
        self.keys_lines_cols = []
        if self.debug or not os.path.exists(os.path.join(__dirname__, "captchadata.json")):
            self._combine_colors()
            self._recipes()
            self._vectors()
            self._svgs()
        else:
            with open(os.path.join(__dirname__, "captchadata.json"), "r", encoding="utf-8") as f:
                captchadata = json.load(f)
                self.options = captchadata[0]
                self.grafical_forms = captchadata[1]

    def _combine_colors(self):
        cont_b = 0
        sass = ""
        for x in self._background_color:
            cont_f = 0
            self.options[x] = x
            t_sass = ""
            cont_b += 1
            t_sass_0 = "".join(["    .fil0\n        fill: ", self._background_color[x], "\n"])
            for y in self._foreground_color:
                self.options[y] = y
                cont_f += 1
                t_sass_1 = "".join([
                    t_sass,
                    "    .fil1\n        fill: ",
                    self._foreground_color[y],
                    "\n",
                    " " * 8,
                    "fill-rule: nonzero",
                    "\n"
                ])
                t_sass = "".join([
                    ".phanterpwa-captchasvg-{0}{1}".format(
                        str(cont_b).zfill(2),
                        str(cont_f).zfill(2)
                    ),
                    "\n", t_sass_0,
                    t_sass_1
                ])
                sass = "\n".join([sass, t_sass])
                t_sass = ""
                self.color_pairs[
                    "phanterpwa-captchasvg-{0}{1}".format(
                        str(cont_b).zfill(2),
                        str(cont_f).zfill(2)
                    )
                ] = [x, y]
        with open(os.path.join(__dirname__, "sass", "captcha.sass"), "w", encoding="utf-8") as f:
            f.write(sass)

    def _recipes(self):
        recs = glob(os.path.join(__dirname__, "recipes", "*.recs"))
        for x in recs:
            with open(x, "r", encoding="utf-8") as f:
                basename = os.path.basename(x)[:-5]
                self.options[basename] = basename
                self.recipes[basename] = str(f.read())

    def _vectors(self):

        clas = glob(os.path.join(__dirname__, "vectors", "*"))
        for x in clas:
            if os.path.isdir(x):
                classification_name = os.path.split(x)[-1]
                self.classification.add(classification_name)
                self.options[classification_name] = classification_name
                vecs = glob(os.path.join(x, "*.recs"))
                for v in vecs:
                    with open(v, "r", encoding="utf-8") as f:
                        basename = os.path.basename(v)[:-5]
                        self.options[basename] = basename
                        self.vectors[(classification_name, basename)] = str(f.read())

    def _svgs(self):
        code = 0
        for c in self.color_pairs.keys():
            for r in self.recipes:
                for v in self.vectors:
                    code += 1
                    self.grafical_forms[code] = list(self.color_pairs[c])
                    self.grafical_forms[code].append(r)
                    self.grafical_forms[code].append(v[0])
                    self.grafical_forms[code].append(v[1])
                    self.grafical_forms[code].append(c)
        with open(os.path.join(__dirname__, "captchadata.json"), "w", encoding="utf-8") as f:
            json.dump([self.options, self.grafical_forms], f, ensure_ascii=True, indent=2)

    @property
    def option(self):
        if not self._option:
            self._choicer()
        return self._option

    @property
    def choice(self):
        if not self._choice:
            self._choicer()
        return self._choice

    @property
    def signature(self):
        if not self._signature:
            self._choicer()
        return self._signature

    def _choicer(self):
        self.keys_attributes = [x for x in self.options.keys()] +\
            [x[1] for x in self.vectors.keys()] +\
            [x[1] for x in self.vectors.keys()]
        self.keys_lines_cols = [x for x in self.grafical_forms.keys()]
        count_attr = len(self.keys_attributes)
        choice = self.keys_attributes[random.randint(0, count_attr - 1)]
        self._choice = choice
        self._opt = []
        random.shuffle(self.keys_lines_cols)
        cont_err = 0
        cont_ok = 0
        self._option = None
        for x in self.keys_lines_cols:
            t_choice = self.grafical_forms[x][:-1]
            if choice in t_choice:
                if cont_ok == 0:
                    self._opt.append(x)
                    self._option = x
                    cont_ok += 1
            else:
                if cont_err < self.num_opt - 1:
                    self._opt.append(x)
                    cont_err += 1
                else:
                    if cont_ok == 1:
                        break
        random.shuffle(self._opt)
        sign_captha = self.serializer.dumps({
            'id_form': self._id,
            'choice': choice,
            'option': self._option
        })
        self._signature = sign_captha

    def check(self, signature, option):
        read_signature = None
        try:
            read_signature = self.serializer.loads(signature)
        except BadSignature:
            read_signature = None
        except SignatureExpired:
            read_signature = None
        else:
            try:
                read_option = self.serializer.loads(option)
            except BadSignature:
                read_option = None
            except SignatureExpired:
                read_option = None
            else:
                if read_option and read_signature:
                    if read_signature['choice'] in self.grafical_forms[str(read_option['option'])][:-1] and\
                        read_signature['id_form'] == self._id:
                        return True
                    else:
                        return False
        return False

    @property
    def html(self):
        question = self.question
        choice = self.choice
        new_dict_t = dict()
        opt = self.options[choice]
        if isinstance(self.translator, Translator):
            for d in self.translator.languages:
                t = self.translator.translator(question, d)
                new_dict_t["_{0}".format(d.lower())] = t.format(option=self.translator.translator(opt, d))
            new_dict_t["_phanterpwa-i18n"] = question.format(option=opt)
            question = SPAN(
                question.format(option=opt),
                **new_dict_t
            )
        else:
            question = question.format(option=opt)
        content = []
        if self.debug:
            token_question = choice
        else:
            token_question = self.signature
        for x in self._opt:
            attrs = {
                "_xmlns": "http://www.w3.org/2000/svg",
                "_xml:space": "preserve",
                "_width": "50px",
                "_height": "50px",
                "_style": "shape-rendering:geometricPrecision; text-rendering:geometricPrecision; " +
                    "image-rendering:optimizeQuality; fill-rule:evenodd; clip-rule:evenodd",
                "_viewBox": "0 0 50 50",
                "_xmlns:xlink": "http://www.w3.org/1999/xlink",
            }
            svg_e = self.grafical_forms[x]
            recipe = svg_e[2]
            sub_folder = svg_e[3]
            vector = svg_e[4]
            attrs["_class"] = svg_e[5]
            svg_recipe = ""
            svg_vector = ""
            if self.debug:
                token_option = str(x)
            else:
                sign_option = self.serializer.dumps({
                    'option': str(x)
                })
                token_option = sign_option
            p = os.path.join(__dirname__, "recipes", "{0}.recs".format(recipe))
            with open(p, 'r', encoding='utf-8') as f:
                svg_recipe = f.read()
            p = os.path.join(__dirname__, "vectors", sub_folder, "{0}.recs".format(vector))
            with open(p, 'r', encoding='utf-8') as f:
                svg_vector = f.read()
            content.append(
                DIV(
                    DIV(
                        DIV(
                            SVG(XML(svg_recipe), XML(svg_vector), **attrs),
                            _class='captcha-option-svg'),
                        _class='captcha-option link',
                        _token_option=token_option,
                        _token_question=token_question,
                        _id_captcha=self._id
                    ),
                    _class='captcha-option-container')
            )
        html = DIV(
            DIV(
                XML(question),
                _class='captcha-question-container'),
            DIV(
                *content,
                _class='captcha-options-container'),
            _class='captcha-container')
        html.sass_file(os.path.join(__dirname__, "sass", "captcha.sass"))
        self._html = html
        return self._html

    @property
    def html_ok(self):
        self._html_ok = DIV(
            DIV(
                XML("".join(['<svg xmlns="http://www.w3.org/2000/svg" xml:space="preserve" width="50px" '
                    'height="50px" style="shape-rendering:geometricPrecision; text-rendering:geometricPrecision;',
                    ' image-rendering:optimizeQuality; fill-rule:evenodd; clip-rule:evenodd" viewBox="0 0 50 50"',
                    ' xmlns:xlink="http://www.w3.org/1999/xlink"><circle class="fil0" cx="25.1541" cy="25.1175" ',
                    'r="24.5529"/><polygon class="fil1" points="14.6544,18.9408 20.4585,26.0181 39.3804,13.6859'
                    ' 43.962,20.7361 21.8877,35.1224 18.7388,37.1752 16.3517,34.266 8.16404,24.2795 "/></svg>'])),
                _class="captcha-ok-svg-container", _id="captcha-ok-svg-container-{0}".format(self._id)),
            _class='captcha-container')
        return self._html_ok


if __name__ == '__main__':
    captcha = Captcha("form", "teste", 5000)
    print(captcha.choice)
    print(captcha.html)



import phanterpwa.frontend.application as application
import phanterpwa.frontend.helpers as helpers
import phanterpwa.frontend.components.auth_user as auth_user
import phanterpwa.frontend.components.left_bar as left_bar
import auto.config as config
import gatehandlers.reqs as reqs
import gatehandlers.testsp as testsp
import gatehandlers.project as project
import gatehandlers.developer as developer
import gatehandlers.fontawesome as fontawesome
import gatehandlers.examples as examples
import gatehandlers.errors as errors
import gatehandlers.home as home
from org.transcrypt.stubs.browser import __pragma__


__pragma__("skip")
jQuery = window = console = WebSocket = __new__ = location = this = confirm = 0
__pragma__("noskip")


gates = {
    'home': home.Index,
    'examples': examples.Index,
    'fontawesome': fontawesome.Index,
    'developer': developer.Index,
    'project': project.Index,
    'test_phanterpwa': testsp.Index,
    'check_requeriments': reqs.Index,
    'examples': examples.Index,
    401: errors.Error_401,
    403: errors.Error_403,
    404: errors.Error_404
}


MyApp = application.PhanterPWA(config.CONFIG, gates)
MyApp.add_component(left_bar.LeftBarMainButton("#layout-main_button-container"))
MyApp.add_component(left_bar.LeftBar("#layout-left_bar-container"))
MyApp.open_current_way()




import phanterpwa.frontend.gatehandler as gatehandler
import phanterpwa.frontend.components.widgets as widgets
import phanterpwa.frontend.helpers as helpers
import plugins.codemirror as codemirror
import phanterpwa.frontend.components.left_bar as left_bar
import gatehandlers.ext_examples.inputs as inputs
import gatehandlers.ext_examples.selects as selects
import gatehandlers.ext_examples.menus as menus

from org.transcrypt.stubs.browser import __pragma__

__pragma__('alias', "jQuery", "$")
__pragma__('skip')
jQuery = sessionStorage = JSON = M = js_undefined = window =\
    __new__ = FormData = console = localStorage = document = this = 0
__pragma__('noskip')

DIV = helpers.XmlConstructor.tagger("div")
LABEL = helpers.XmlConstructor.tagger("label")
I = helpers.XmlConstructor.tagger("i")
H2 = helpers.XmlConstructor.tagger("h2")
FORM = helpers.XmlConstructor.tagger("form")
SPAN = helpers.XmlConstructor.tagger("span")
IMG = helpers.XmlConstructor.tagger("img", True)
HR = helpers.XmlConstructor.tagger("hr", True)
UL = helpers.XmlConstructor.tagger("ul")
STRONG = helpers.XmlConstructor.tagger("strong")
LI = helpers.XmlConstructor.tagger("li")
INPUT = helpers.XmlConstructor.tagger("input", True)
BR = helpers.XmlConstructor.tagger("br", True)
P = helpers.XmlConstructor.tagger("p")
A = helpers.XmlConstructor.tagger("a")
I18N = helpers.I18N
CONCATENATE = helpers.CONCATENATE
XML = helpers.XML
XSECTION = helpers.XSECTION


class Index(gatehandler.Handler):
    def initialize(self):
        if self.request.get_arg(0) == "inputs":
            html = CONCATENATE(
                DIV(
                    DIV(
                        DIV(
                            DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                            DIV("INPUT", _class="phanterpwa-breadcrumb"),
                            _class="phanterpwa-breadcrumb-wrapper"
                        ),
                        _class="p-container"),
                    _class='title_page_container card'
                ),
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                widgets.Preloaders("preload_android"),
                                _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                            ),
                            _id="projects_container"),
                        _class="card p-row e-padding_10"
                    ),
                    _id="input-container-example",
                    _class="phanterpwa-container p-container"
                )
            )
            html.html_to("#main-container")
            setTimeout(lambda: inputs.Inputs(self), 30)
        elif self.request.get_arg(0) == "selects":
            html = CONCATENATE(
                DIV(
                    DIV(
                        DIV(
                            DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                            DIV("SELECT", _class="phanterpwa-breadcrumb"),
                            _class="phanterpwa-breadcrumb-wrapper"
                        ),
                        _class="p-container"),
                    _class='title_page_container card'
                ),
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                widgets.Preloaders("preload_android"),
                                _style="width:100%; text-align: center; padding-top: 100px; padding-bottom: 100px;"
                            ),
                            _id="projects_container"),
                        _class="card p-row e-padding_10"
                    ),
                    _id="selects-container-example",
                    _class="phanterpwa-container p-container"
                )
            )
            html.html_to("#main-container")
            setTimeout(lambda: selects.Selects(self), 30)
        elif self.request.get_arg(0) == "menus":
            self.menus = menus.MenuBox(self)
        elif self.request.get_arg(0) == "others":
            self.others = Others(self)
        else:
            html = CONCATENATE(
                DIV(
                    DIV(
                        DIV(
                            DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                            _class="phanterpwa-breadcrumb-wrapper"
                        ),
                        _class="p-container"),
                    _class='title_page_container card'
                ),
                DIV(
                    DIV(
                        DIV(
                            DIV(
                                DIV(
                                    I(
                                        **{
                                            "_class": "fas fa-edit promo-icon",
                                        }
                                    ),
                                    H2("Inputs", _class="promo-title"),
                                    **{
                                        "_class": "link",
                                        "_phanterpwa-way": "examples/inputs"
                                    }
                                ),
                                **{"_class": "promo-container"}
                            ),
                            _class='p-col w1p100 w3p50 w4p25'
                        ),
                        DIV(
                            DIV(
                                DIV(
                                    I(
                                        **{
                                            "_class": "far fa-caret-square-down promo-icon",
                                        }
                                    ),
                                    H2("Selects", _class="promo-title"),
                                    **{
                                        "_class": "link",
                                        "_phanterpwa-way": "examples/selects"
                                    }
                                ),
                                **{"_class": "promo-container"}
                            ),
                            _class='p-col w1p100 w3p50 w4p25'),
                        DIV(
                            DIV(
                                DIV(
                                    I(
                                        **{
                                            "_class": "fas fa-ellipsis-v  promo-icon",
                                        }
                                    ),
                                    H2("Menus", _class="promo-title"),
                                    **{
                                        "_class": "link",
                                        "_phanterpwa-way": "examples/menus"
                                    }
                                ),
                                **{"_class": "promo-container"}
                            ),
                            _class='p-col w1p100 w3p50 w4p25'),
                        DIV(
                            DIV(
                                DIV(
                                    I(
                                        **{
                                            "_class": "fas fa-file-signature  promo-icon",
                                        }
                                    ),
                                    H2("Others", _class="promo-title"),
                                    **{
                                        "_class": "link",
                                        "_phanterpwa-way": "examples/others"
                                    }
                                ),
                                **{"_class": "promo-container"}
                            ),
                            _class='p-col w1p100 w3p50 w4p25'),
                        DIV(
                            DIV(
                                DIV(
                                    I(
                                        **{
                                            "_class": "fas fa-flag promo-icon",
                                        }
                                    ),
                                    H2("Font Awesome", _class="promo-title"),
                                    **{
                                        "_class": "link",
                                        "_phanterpwa-way": "fontawesome"
                                    }
                                ),
                                **{"_class": "promo-container"}
                            ),
                            _class='p-col w1p100 w3p50 w4p25'),
                        _class='p-row card e-padding_20'
                    ),

                    _class="phanterpwa-container p-container"
                )
            )
            html.html_to("#main-container")
        BackButton1 = left_bar.LeftBarButton(
            "back_home",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{"_phanterpwa-way": "home",
                "position": "top",
                "ways": [lambda r: True if r == "examples" else False]}
        )
        BackButton2 = left_bar.LeftBarButton(
            "back_examples",
            "Voltar",
            I(_class="fas fa-arrow-circle-left"),
            **{"_phanterpwa-way": "examples",
                "position": "top",
                "ways": [lambda r: True if r.startswith("examples/") else False]}
        )
        window.PhanterPWA.Components['left_bar'].add_button(BackButton1)
        window.PhanterPWA.Components['left_bar'].add_button(BackButton2)



class Others():
    def __init__(self, gatehandler):
        self.gatehandler = gatehandler
        html = CONCATENATE(
            DIV(
                DIV(
                    DIV(
                        DIV("COMPONENTES", _class="phanterpwa-breadcrumb"),
                        DIV("TEXTAREA", _class="phanterpwa-breadcrumb"),
                        _class="phanterpwa-breadcrumb-wrapper"
                    ),
                    _class="p-container"),
                _class='title_page_container card'
            ),
            DIV(
                DIV(
                    DIV(
                        H2(I18N("Textboxs")),
                        HR(),
                        XSECTION(
                            LABEL("Type: Password"),
                            DIV(
                                widgets.Textarea(
                                    "textare1",
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("Type: Password"),
                            DIV(
                                widgets.Textarea(
                                    "textare2",
                                    label="Label Widget",
                                    wear="shadows"
                                ),
                                _class="component_shadows"
                            ),
                        ),
                        XSECTION(
                            LABEL("Type: Password"),
                            DIV(
                                widgets.Textarea(
                                    "textare3",
                                    label="Label Widget",
                                    wear="elegant"
                                ),
                                _class="component_shadows"
                            ),
                        ),
                        H2(I18N("Dropdown")),
                        HR(),
                        XSECTION(
                            LABEL("Value: True"),
                            DIV(
                                DIV(
                                    DIV(
                                        widgets.MenuBox(
                                            "drop1",
                                            xml_menu=UL(
                                                LI(SPAN("Editar", _class="botao_editar_socio")),
                                                LI(SPAN("Visualizar", _class="botao_visualizar_socio")),
                                                _class='dropdown-content'
                                            )
                                        ),
                                        _class="p-col w1p10"
                                    ),
                                    DIV(
                                        " ",
                                        _class="p-col w1p80"
                                    ),
                                    DIV(
                                        widgets.MenuBox(
                                            "drop2",
                                            xml_menu=UL(
                                                LI(SPAN("Editar", _class="botao_editar_socio")),
                                                LI(SPAN("Visualizar", _class="botao_visualizar_socio")),
                                                _class='dropdown-content'
                                            )
                                        ),
                                        _class="p-col w1p10"
                                    ),
                                    _class="p-row"
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        H2(I18N("Checkbox")),
                        HR(),
                        XSECTION(
                            LABEL("Value: True"),
                            DIV(
                                widgets.CheckBox(
                                    "check1",
                                    value=True,
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("Value: False"),
                            DIV(
                                widgets.CheckBox(
                                    "check2",
                                    value=False,
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("Value: False"),
                            DIV(
                                widgets.CheckBox(
                                    "check3",
                                    value=False,
                                    label="Label Widget",
                                    wear="elegant"
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        H2(I18N("Radiobox")),
                        HR(),
                        XSECTION(
                            LABEL("Value: True"),
                            DIV(
                                widgets.RadioBox(
                                    "radio1",
                                    value=True,
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("Value: False"),
                            DIV(
                                widgets.RadioBox(
                                    "radio2",
                                    value=False,
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("Value: False"),
                            DIV(
                                widgets.RadioBox(
                                    "radio4",
                                    value=False,
                                    label="Label Widget",
                                    name="my_radio"
                                ),
                                widgets.RadioBox(
                                    "radio5",
                                    value=False,
                                    label="Label Widget",
                                    name="my_radio"
                                ),
                                widgets.RadioBox(
                                    "radio6",
                                    value=False,
                                    label="Label Widget",
                                    name="my_radio"
                                ),
                                widgets.RadioBox(
                                    "radio7",
                                    value=False,
                                    label="Label Widget",
                                    name="my_radio"
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        H2(I18N("ListString")),
                        HR(),
                        XSECTION(
                            LABEL("Value: [\"Value01\", \"Value02\", \"Value03\"]"),
                            DIV(
                                widgets.ListString(
                                    "liststring1",
                                    value=["Value01", "Value02", "Value03"],
                                    label="Label Widget",
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        XSECTION(
                            LABEL("data_set: [\"Value04\", \"Value05\"] and editable: False"),
                            DIV(
                                widgets.ListString(
                                    "liststring2",
                                    value=["Value01", "Value02", "Value03"],
                                    label="Label Widget",
                                    data_set=["Value04", "Value05"],
                                    editable=False
                                ),
                                _class="component_materialize"
                            ),
                        ),
                        _class="card e-padding_20"
                    ),
                    _class="new-container"
                ),
                _class="phanterpwa-container p-container"
            )
        )
        jQuery("#main-container").html(html.jquery())





"""
Title: XmlConstructor

Author: PhanterJR<junior.conex@gmail.com>

License: MIT

Coding: utf-8

Html to XmlConstructor
"""

from html.parser import HTMLParser
from .xmlconstructor import XmlConstructor
from .helpers import CONCATENATE, XCOMMENT


class _Tag(XmlConstructor):
    def __init__(self, tag, void, is_closed):
        XmlConstructor.__init__(self, tag, void)
        self._is_closed = False

    @property
    def is_closed(self):
        return self._is_closed

    @is_closed.setter
    def is_closed(self, v):
        if self.void is True:
            self._is_closed = True
        else:
            if isinstance(v, bool):
                self._is_closed = v
            else:
                raise TypeError("is_closed must be bool, given: {0}".format(type(v)))


class HtmlToXmlConstructor(CONCATENATE, HTMLParser):
    """The XmlConstructor writes html, this object already does the opposite, with it it is possible with html to
    write XmlConstructor.

    At instantiation we have the following parameters

    :param strhtml: String with html code.
    :param filter_empty_content: Filters empty elements when True (False)
    :param filter_comments: Filter comments elements when True (True)
    """

    def __init__(self, strhtml, filter_empty_content=False, filter_comments=True):
        self.void_tags = ["br", "area", "base", "col", "embed", "hr", "img",
            "input", "link", "meta", "param", "source", "track", "wbr"]
        self.strhtml = strhtml
        self.opened_el = []
        CONCATENATE.__init__(self, "")
        HTMLParser.__init__(self)
        self.content = []
        self.filter_empty_content = filter_empty_content
        self.HTMLdoctypedeclaration = ""
        self.feed(strhtml)

    def handle_starttag(self, tag, attrs):
        """Manipulate the initial tag and its attributes, see html.parse.HTMLParser for more details.
        """
        attr_tag = {}
        for attr in attrs:
            attr_tag["_%s" % attr[0]] = True if attr[1] is None else attr[1]
        el = _Tag(tag, True if tag in self.void_tags else False, False)
        el.attributes = attr_tag
        if all([tag == "html",
            self.HTMLdoctypedeclaration,
            self.HTMLdoctypedeclaration != "<!DOCTYPE html>"]):
            el.before_xml = self.HTMLdoctypedeclaration
        if self.opened_el:
            last_el = self.opened_el[-1]
            if not last_el.is_closed:
                content = list(last_el.content)
                content.append(el)
                last_el.content = content
        else:
            self.append(el)
        if tag not in self.void_tags:
            self.opened_el.append(el)

    def handle_endtag(self, tag):
        """Manipulate the end tag, see html.parse.HTMLParser for more details.
        """
        if tag not in self.void_tags:
            last_el = self.opened_el[-1]
            if not last_el.is_closed:
                last_el.is_closed = True
                self.opened_el.pop(-1)

    def handle_data(self, data):
        """Manipulate the data, see html.parse.HTMLParser for more details.
        """
        just_data = set([" ", "\n", "\t", "\r"])
        set_data = set(data)
        if self.opened_el:
            last_el = self.opened_el[-1]
            if not last_el.is_closed:
                content = list(last_el.content)
                if (set_data.union(just_data) == just_data):
                    if not self.filter_empty_content:
                        content.append(data)
                else:
                    content.append(data)
                last_el.content = content
        else:
            if set_data.union(just_data) == just_data:
                if not self.filter_empty_content:
                    self.append(data)
            else:
                self.append(data)

    def handle_comment(self, data):
        """Manipulate the comments, see html.parse.HTMLParser for more details.
        """
        if self.opened_el:
            last_el = self.opened_el[-1]
            if not last_el.is_closed:
                content = list(last_el.content)
                content.append(XCOMMENT(data))
                last_el.content = content
        else:
            self.append(XCOMMENT(data))

    def handle_decl(self, data):
        """Manipulate the declarations, see html.parse.HTMLParser for more details.
        """
        self.HTMLdoctypedeclaration = "<!{0}>".format(data.strip())

    def xmlconstructor_code(self, add_imports: bool=False, phanterpwa_helpers: bool=False, instance_name: str="html",
            translate: bool=False) -> str:
        """Returns the generated source code, does the same as the source_code method with the difference of omit
        the CONCATENATE root tag when necessary.

        see the documentation for the source_code method of phanterpwa.xmlconstructor.XmlConstructor for more details.

        :param add_imports: add necessary imports in code This arguments
            will take effect if add_imports is true.
        :param phanterpwa_helpers: user phanter.helpers import.
        :param instance_name: The generated code is assigned to a variable
            with the name assigned here.
        :param translate: If True, the generated source_code strings will
            be translated.

        Example:
            >>> from phanterpwa.reversexml import HtmlToXmlConstructor
            >>> html_to_xmlconstructor_instance = HtmlToXmlConstructor("<html><head><meta charset=\"UTF-8\"></head><body><nav class=\"navbar\"><buttom>start</buttom></nav><main id=\"my_content\"><div class=\"row\"><div>my content</div></div></main></body></html>")
            >>> print(html_to_xmlconstructor_instance.xmlconstructor_code())
            HTML(
                HEAD(
                    META(
                        _charset='UTF-8'
                    )
                ),
                BODY(
                    NAV(
                        BUTTOM(
                            'start'
                        ),
                        _class='navbar'
                    ),
                    MAIN(
                        DIV(
                            DIV(
                                'my content'
                            ),
                            _class='row'
                        ),
                        _id='my_content'
                    )
                )
            )
            >>> html_to_xmlconstructor_instance = HtmlToXmlConstructor("<div data-dict=\"i am in dict\" class=\"my_class\">content1</div><div class=\"my_class\">content2</div>")
            >>> print(html_to_xmlconstructor_instance.xmlconstructor_code())
            CONCATENATE(
                DIV(
                    'content1',
                    **{
                        '_data-dict': 'i am in dict',
                        '_class': 'my_class'
                    }
                ),
                DIV(
                    'content2',
                    _class='my_class'
                )
            )
        """
        if len(self) == 1:
            child = self[0]
            child._indent_level = 0
            child._parent = None
            return child.source_code(add_imports, phanterpwa_helpers, instance_name, translate)
        else:
            return self.source_code(add_imports, phanterpwa_helpers, instance_name, translate)


def force_minify_string_content(target: XmlConstructor) -> XmlConstructor:
    """Forces the minification of the string content by removing multiple empty spaces, line breaks, tabs replacing
    with a single space.

    :param target: XmlConstructor (or HtmlToXmlConstructor) instance

    Example:
        >>> from phanterpwa.reversexml import (HtmlToXmlConstructor, force_minify_string_content)
        >>> sample = '''
        ... <html>
        ...     <head>
        ...         <meta charset="utf-8">
        ...     </head>
        ...     <body>
        ...         Forces      the       minification
        ...         of the     string content       by
        ...         removing       multiple     empty
        ...         spaces,      line       breaks,
        ...         tabs      replacing     with a
        ...         single    space.
        ...     </body>
        ... </html>
        ... '''
        >>> html_to_xmlconstructor_instance = HtmlToXmlConstructor(sample)
        >>> print(html_to_xmlconstructor_instance.xml())
        <html>
            <head>
                <meta charset="utf-8">
            </head>
            <body>
                Forces      the       minification
                of the     string content       by
                removing       multiple     empty
                spaces,      line       breaks,
                tabs      replacing     with a
                single    space.
            </body>
        </html>
        >>> force_minify_string_content(html_to_xmlconstructor_instance)
        >>> print(html_to_xmlconstructor_instance.xml())
        <html><head><meta charset="utf-8"></head><body> Forces the minification of the string content by removing multiple empty spaces, line breaks, tabs replacing with a single space. </body></html>
        >>> print(html_to_xmlconstructor_instance.xmlconstructor_code())
        HTML(
            HEAD(
                META(
                    _charset='utf-8'
                )
            ),
            BODY(
                ' Forces the minification of the string content by removing multiple empty spaces, line breaks, tabs replacing with a single space. '
            )
        )
    """
    if isinstance(target, XmlConstructor):
        def f(r):
            new_content = []
            skippers = [" ", "\n", "\t", "\r"]
            for x in r:
                if isinstance(x, XmlConstructor):
                    f(x)
                    new_content.append(x)
                elif isinstance(x, str):
                    set_str = set(x)
                    if set_str.union(set(skippers)) == set(skippers):
                        pass
                    else:
                        if x:
                            i_std = ""
                            f_str = ""
                            if x[0] in skippers:
                                i_std = " "
                            if x[-1] in skippers:
                                f_str = " "
                            n_v = " ".join([z.strip() for z in x.split(" ") if z not in skippers and z])
                            new_content.append("".join([i_std, n_v, f_str]))
                else:
                    new_content.append(x)
            r.content = new_content
        f(target)
        return target
    else:
        raise ValueError("The target must be XmlConstructor instance.")

